function data = data_gui3( cb, arg1 )
% data gui library, written by sascha meykopff, 2011-2014, desy hamburg

    if ~(  (nargout==0 && nargin==1) || (nargout==1 && nargin==0) || (nargout==0 && nargin==2) )
        throwError('call data=data_gui3(); or data_gui3(@your_init_callback); or data_gui3(@your_callback, your_arg)');
    end
    try
        cmdWinDoc = com.mathworks.mde.cmdwin.CmdWinDocument.getInstance;
        listeners = cmdWinDoc.getDocumentListeners;
        isdesktop = false;
        if length(listeners) > 1; isdesktop = true; end
    catch %#ok<CTCH>
        exc = lasterror; %#ok<LERR>
        error = exc.message;
        display([ 'exception in data_gui3: ' error ]);
        isdesktop = true;
    end
    
    data = struct();
    data.CALL = struct();
    data.GUILIB = struct();
    data.GUILIB.dataRegistry = cell(6,0); 
    data.GUILIB.dataEntries = 0;
    data.GUILIB.errorCB = [];
    data.GUILIB.isDesktop = isdesktop;
    data.GUILIB.revision = '$Rev: 166 $';
    data.GUILIB.axeshandles = struct();
    data.GUILIB.layoutData = cell(0,2);
    data.GUILIB.matlabVersion = getMatlabVersion();

    rev_indx = strfind(data.GUILIB.revision,' ');
    if length(rev_indx) == 2
        rev = str2double(data.GUILIB.revision(rev_indx(1)+1:rev_indx(2)-1));
        if ~isnan(rev)
            data.GUILIB.revisionNumber = rev;
        end
    end
    
    data.CALL.addGui          = @addGui;
    data.CALL.addAxes         = @addAxes;
    data.CALL.addCheckbox     = @addCheckbox;
    data.CALL.addDeref        = @addDeref;
    data.CALL.addEdittext     = @addEdittext;
    data.CALL.addFigure       = @addFigure;
    data.CALL.addLabel        = @addLabel;
    data.CALL.addListbox      = @addListbox;
    data.CALL.addMenuitem     = @addMenuItem;    
    data.CALL.addPanel        = @addPanel;
    data.CALL.addPopupbox     = @addPopupbox;
    data.CALL.addPushbutton   = @addPushbutton;
    data.CALL.addRadiogroup   = @addRadioGroup;
    data.CALL.addSlider       = @addSlider;
    data.CALL.addStatictext   = @addStaticText;
    data.CALL.addTable        = @addTable;
    data.CALL.addTimer        = @addTimer;
    data.CALL.addTogglebutton = @addTogglebutton;
    data.CALL.addTabGroup     = @addTabGroup;
    data.CALL.addTree         = @addTree;
    data.CALL.addTreeDyn      = @addTreeDyn;
    data.CALL.closeFigure     = @closeFigure;
    
    data.CALL.expandTree      = @expandTreeDyn;
    data.CALL.wrapCallback    = @callbackWrapper;
    
    data.CALL.addData         = @addData;
    data.CALL.insertEntry     = @insertEntry;
    data.CALL.storeData       = @storeUserData;
    data.CALL.resolveTag      = @resolveHandle;
    data.CALL.resolveIndex    = @findNameByIndex;
    data.CALL.updateEntry     = @manualUpdateEntry;
    data.CALL.loadData        = @loadData;
    data.CALL.saveData        = @saveData;
    data.CALL.defaultValue    = @setDefaultValue;
    data.CALL.updateConstrain = @updateConstrain;
    data.CALL.addDataErrorCB  = @registerDataErrorCallback;

    data.CALL.addLayout       = @addLayout;
    data.CALL.layoutTop       = @layoutTop;
    data.CALL.layoutBottom    = @layoutBottom;
    data.CALL.layoutVCenter   = @layoutVCenter;
    data.CALL.layoutHCenter   = @layoutHCenter;
    data.CALL.layoutLeft      = @layoutLeft;
    data.CALL.layoutRight     = @layoutRight;
    data.CALL.layoutPanel     = @layoutPanel;
    data.CALL.layoutShrink    = @layoutShrink;
    data.CALL.layoutSquare    = @layoutSquare;
    data.CALL.manualResize    = @manualResize;
    
    data.CALL.dumpGuiRegistry = @dumpGuiRegistry; % debug only
       
    if nargout==0
        switch nargin
            case 1
                [data,redraw] = cb(data, []); % user callback
            case 2
                [data,redraw] = cb(data, [], arg1); % user callback
        end
        if ~isfield( data.GUILIB, 'figures' )
            return % no figure created
        end
        if size(data.GUILIB.figures,2) == 0
            return
        end
        if ~ishandle(data.GUILIB.figures)
            return % handle not valid
        end
        if isempty(redraw); 
            data = doGuiUpdate( data );
            for i=1:size(data.GUILIB.figures,2); % force resize
                data = manualResize( data, data.GUILIB.figures(i) );
            end
        end;
        storeUserData(data);
        if isdesktop==false
            uiwait(data.GUILIB.figures(1));
            quit;
        end
    end
end

% public gui api

function data = addGui( data, input )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    for i=1:size(input,1)
        in = input{i,1};
        type = lower( in{1,1} );
        tagname = in{1,2};
        switch type
            case 'axes'
                data = addAxes(data, tagname, in{1,3}, cellShift(in,3) );
            case 'checkbox'
                data = addCheckbox(data, tagname, in{1,3}, in{1,4}, cellShift(in,4) );
            case 'deref'
                data = addDeref(data, tagname, in{1,3} );
            case 'edittext'
                data = addEdittext(data, tagname, in{1,3}, in{1,4}, cellShift(in,4) );
            case 'figure'
                data = addFigure(data, tagname, cellShift(in,2) );
            case 'label'
                data = addLabel(data, tagname, in{1,3}, in{1,4}, cellShift(in,4) );
            case 'listbox'
                data = addListbox(data, tagname, in{1,3}, in{1,4}, in{1,5}, cellShift(in,5) );
            case 'menuitem'
                data = addMenuItem(data, tagname, in{1,3}, in{1,4}, cellShift(in,4) );
            case 'panel'
                data = addPanel(data, tagname, in{1,3}, cellShift(in,3) );
            case 'popupbox'
                data = addPopupbox(data, tagname, in{1,3}, in{1,4}, in{1,5}, cellShift(in,5) );
            case 'pushbutton'
                data = addPushbutton(data, tagname, in{1,3}, in{1,4}, cellShift(in,4) );
            case 'radiogroup'
                data = addRadioGroup(data, tagname, in{1,3}, in{1,4}, in{1,5}, cellShift(in,5) );
            case 'slider'
                data = addSlider(data, tagname, in{1,3}, in{1,4}, cellShift(in,4) );
            case 'statictext'
                data = addStaticText(data, tagname, in{1,3}, in{1,4}, cellShift(in,4) );
            case 'table'
                data = addTable(data, tagname, in{1,3}, in{1,4}, cellShift(in,4));
            case 'timer'
                data = addTimer(data, tagname, in{1,3}, cellShift(in,3) );
            case 'togglebutton'
                data = addTogglebutton(data, tagname, in{1,3}, in{1,4}, in{1,5}, cellShift(in,5) );
            case 'tabgroup'
                data = addTabGroup(data, tagname, in{1,3}, in{1,4}, in{1,5}, cellShift(in,5) );
            case 'tree'
                data = addTree( data, tagname, in{1,3}, in{1,4}, in{1,5}, cellShift(in,5) );
            case 'treedyn'
                data = addTreeDyn( data, tagname, in{1,3}, in{1,4}, in{1,5}, cellShift(in,5) );
            otherwise
                display(in);
                throwError([ 'unknown type: ' type]);
        end
    end
end

function [ data,axe ] = addAxes( data, tagname, parent, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);
    axe = axes( 'parent', parent );
    data = storeGuiHandle(data, tagname, axe);
    data.GUILIB.axeshandles.(tagname) = struct('figure', parent, 'inset', [1 2 3 4], 'handle', axe );
    set_dummy_pos( parent, axe );
    data = storeUpdateEntry( data, axe, [{'tag'} {tagname} 'units', 'pixels' {'Interruptible'} {'off'} info ] );
end

function [ data,timr ] = addTimer( data, tagname, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if ~isfield(data.GUILIB,'timerRegistry')
        data.GUILIB.timerRegistry = cell(2,0);
        data.GUILIB.timerEntries = 0;
    end
    checkTagnameUnique(data, tagname);     
    idx = findEntryIndexByName(data, variable );
    if isempty(idx); throwError([ 'variable ' variable ' not found']); end;
    
    timr = timer('TimerFcn',{@timerCallback, data.GUILIB.figures(1), idx},  'tag',tagname, 'BusyMode', 'drop'); %, );
    data = storeGuiHandle(data, tagname, timr);
    data.GUILIB.timerEntries = data.GUILIB.timerEntries + 1;
    data.GUILIB.timerRegistry{1,data.GUILIB.timerEntries} = timr;
    data = storeUpdateEntry( data, timr, [{'startstop'} {variable} info]);
end

function [ data,listbox ] = addListbox( data, tagname, parent, variable, text, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);
    dataIndex = findEntryIndexByName(data, variable );
    if isempty(dataIndex); throwError([ 'variable ' variable ' not found']); end;
    listbox = uicontrol( parent, 'style', 'listbox', 'callback', {@callRedirect @uicontrolValueChanged dataIndex} );
    set_dummy_pos( parent, listbox );
    data = storeGuiHandle(data, tagname, listbox);
    data = storeUpdateEntry( data, listbox, [  {'tag'} {tagname} {'value'} {variable} {'string'} { text } {'Interruptible'} {'off'} info ] );
end

function [ data,toggle ] = addTogglebutton( data, tagname, parent, variable, text, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    dataIndex = findEntryIndexByName(data, variable );
    checkTagnameUnique(data, tagname);     
    if isempty(dataIndex); throwError([ 'variable ' variable ' not found']); end;
    constrain = getConstrain(data, dataIndex);
    toggle = uicontrol( parent, 'style', 'togglebutton', 'callback', {@callRedirect @uicontrolValueChanged dataIndex} );
    data = storeGuiHandle(data, tagname, toggle);
    if isnumeric(constrain)
        info = [ {'tag'} {tagname} {'string'} { text } { 'value' } {variable} {'min'} {constrain(1)} {'max'} {constrain(2)} {'Interruptible'} {'off'} info ];
    else
        info = [ {'tag'} {tagname} {'string'} { text } { 'value' } {variable} {'Interruptible'} {'off'} info ];
    end
    set_dummy_pos( parent, toggle );
    data = storeUpdateEntry( data, toggle, info );
end

function [ data,popup ] = addPopupbox( data, tagname, parent, variable, text, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);    
    dataIndex = findEntryIndexByName(data, variable );
    if isempty(dataIndex); throwError([ 'variable ' variable ' not found']); end;
    popup = uicontrol( parent, 'style', 'popupmenu', 'callback', {@callRedirect @uicontrolValueChanged dataIndex} );
    set_dummy_pos( parent, popup );
    data = storeGuiHandle(data, tagname, popup);
    data = storeUpdateEntry( data, popup, [ {'tag'} {tagname} {'value'} {variable} {'string'} { text } {'Interruptible'} {'off'} info ] );
end

function [ data,slider ] = addSlider( data, tagname, parent, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    dataIndex = findEntryIndexByName(data, variable );
    if isempty(dataIndex); throwError([ 'variable ' variable ' not found']); end;
    checkTagnameUnique(data, tagname);     
    slider = uicontrol( parent, 'style', 'slider', 'callback', {@callRedirect @uicontrolValueChanged dataIndex} );
    data = storeGuiHandle(data, tagname, slider);
    constrain = getConstrain(data, dataIndex);
    col = getBackgroundColorFromHandle( parent );
    if isnumeric(constrain) && ~isempty(constrain)
        info = [ {'tag'} {tagname} { 'value' } {variable} {'min'} {constrain(1)} {'max'} {constrain(2)} {'Interruptible'} {'off'}  {'backgroundcolor'} {col} info ];
    else
        info = [ {'tag'} {tagname} { 'value' } {variable} {'Interruptible'} {'off'}  {'backgroundcolor'} {col} info ];
    end
    set_dummy_pos( parent, slider );
    data = storeUpdateEntry( data, slider, info );
end

function [ data,table ] = addTable( data, tagname, parent, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);    
    entryIndex = findEntryIndexByName(data, variable );
    if isempty(entryIndex); throwError([ 'variable ' variable ' not found']); end;
    table = uitable( parent );
    if isa(table,'javahandle_withcallbacks.com.mathworks.hg.peer.UitablePeer')
        error = 'This old Matlab version can''t handle tables!';
        errordlg(error);
        throwError(error);
    end
    data = storeGuiHandle(data, tagname, table);    
    set(table, 'CellEditCallback',  {@callRedirect @tableDataChanged entryIndex} );
    data = storeUpdateEntry( data, table, [ {'tag'} {tagname} {'data'}, {variable} {'Interruptible'} {'off'} info] );
    set_dummy_pos( parent, table );
end

function [ data,edit] = addEdittext( data, tagname, parent, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);     
    entryIndex = findEntryIndexByName(data, variable );
    if isempty(entryIndex); throwError([ 'variable ' variable ' not found']); end;
    edit = uicontrol( 'style', 'edit', 'parent', parent, 'callback', {@callRedirect @edittextChanged entryIndex} );
    data = storeGuiHandle(data, tagname, edit);
    if nargin<5; info=[]; end
    data = storeUpdateEntry( data, edit, [ {'tag'} {tagname} {'string'}, {variable} {'Interruptible'} {'off'} {'backgroundcolor'} {'white'} info] );
    set_dummy_pos( parent, edit );
end

function [ data,text ] = addStaticText( data, tagname, parent, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);     
    col = getBackgroundColorFromHandle( parent );
    entryIndex = findEntryIndexByName(data, variable );
    if isempty(entryIndex)
        text = uicontrol( 'style', 'text', 'parent', parent, 'string', variable );
        data = storeUpdateEntry( data, text, [ {'tag'} {tagname} {'Interruptible'} {'off'} {'backgroundcolor'} {col} info] );
    else
        text = uicontrol( 'style', 'text', 'parent', parent );
        data = storeUpdateEntry( data, text, [ {'tag'} {tagname} {'string'}, {variable} {'Interruptible'} {'off'} {'backgroundcolor'} {col} info] );
    end
    set_dummy_pos( parent, text );
    data = storeGuiHandle(data, tagname, text);
end

function [ data,menu ] = addMenuItem( data, tagname, parent, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);     
    idx = findEntryIndexByName(data, variable );
    if ~isempty(idx)
        menu = uimenu( 'Parent', parent, 'callback', {@callRedirect @pushbuttonPressed idx} );
        data = storeUpdateEntry( data, menu, [ {'tag'} {tagname} {'label'}, {variable} {'Interruptible'} {'off'} info] );
    else
        menu = uimenu( 'Parent', parent, 'label', variable, 'tag', tagname );
        if ~isempty(info)
            data = storeUpdateEntry( data, menu, [ {'tag'} {tagname} {'Interruptible'} {'off'} info] );
        end
    end
    data = storeGuiHandle(data, tagname, menu);
end

function [ data,panel ] = addPanel( data, tagname, fig, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    fig = resolveHandle( data, fig );
    checkTagnameUnique(data, tagname);    
    col = getBackgroundColorFromHandle( fig );
    panel = uipanel( fig );
    set_dummy_pos( fig, panel );
    data = storeUpdateEntry( data, panel, [ {'tag'} {tagname} {'Interruptible'} {'off'} {'backgroundcolor'} {col} info] );
    data = storeGuiHandle(data, tagname, panel);
end

function [ data,cbx ] = addCheckbox( data, tagname, fig, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    fig = resolveHandle( data, fig );
    checkTagnameUnique(data, tagname);     
    dataIndex = findEntryIndexByName(data, variable );
    if isempty(dataIndex); throwError([ 'variable ' variable ' not found']); end;
    constrain = getConstrain(data, dataIndex);
    cbx = uicontrol( fig, 'style', 'checkbox', 'callback', {@callRedirect @uicontrolValueChanged dataIndex} );
    col = getBackgroundColorFromHandle( fig );
    if isnumeric(constrain)
        info = [ {'tag'} {tagname} { 'value' } {variable} {'min'} {constrain(1)} {'max'} {constrain(2)} {'Interruptible'} {'off'} {'backgroundcolor'} {col} info ];
    else
        info = [ {'tag'} {tagname} { 'value' } {variable} {'Interruptible'} {'off'} {'backgroundcolor'} {col} info ];
    end
    set_dummy_pos( fig, cbx );
    data = storeGuiHandle(data, tagname, cbx);    
    data = storeUpdateEntry( data, cbx, info );
end

function [ data,btn ] = addPushbutton( data, tagname, fig, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    fig = resolveHandle( data, fig );
    checkTagnameUnique(data, tagname);     
    idx = findEntryIndexByName(data, variable );
    if ~isempty(idx)
        btn = uicontrol( fig, 'style', 'pushbutton', 'callback', {@callRedirect @pushbuttonPressed idx} );
        info = [ {'tag'} {tagname} {'String'} {variable} {'Interruptible'} {'off'} info ];
    else
        btn = uicontrol( fig, 'style', 'pushbutton' );
        info = [ {'tag'} {tagname} {'String'} {variable} {'Interruptible'} {'off'} info ];
    end
    set_dummy_pos( fig, btn );
    data = storeGuiHandle(data, tagname, btn);    
    data = storeUpdateEntry( data, btn, info );
end

function [data,fig] = addFigure( data, tagname, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if ~isfield( data.GUILIB, 'figures' ) % setup figures list
        data.GUILIB.figures = [];
        data.GUILIB.figPosEntry = [];
        figurecnt = 0;
    else
        figurecnt = size(data.GUILIB.figures,2);
        checkTagnameUnique(data, tagname);
    end
    figurecnt = figurecnt + 1;
    fig = figure('CloseRequestFcn',@closeFigureCallback);
    moveFigureToFront(fig);
    data.GUILIB.figures(figurecnt) = fig; % store figure handle in figure list
    data.GUILIB.figPosEntry(figurecnt) = 0; % store data entry of position property, we update the data entry after resize cb
    if nargin==2; info=[]; end
    data = storeUpdateEntry( data, data.GUILIB.figures(figurecnt), [{'tag'} {tagname} {'Interruptible'} {'off'} info] ); % include figure in update list
    data = storeGuiHandle(data, tagname, fig);
end

function [data,radiogroup] = addRadioGroup( data, tagname, fig, variable, radios, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    fig = resolveHandle( data, fig );
    checkTagnameUnique(data, tagname);     
    idx = findEntryIndexByName(data, variable );
    if isempty(idx); throwError([ 'variable ' variable ' not found']); end;
    group = uibuttongroup(fig);
    set_dummy_pos( fig, group );
    data = storeGuiHandle(data, tagname, group);

    data = storeUpdateEntry( data, group, ...
        [ {'tag'} {tagname} { 'SelectionChangeFcn' } {{ @callRedirect @radioSelectionChanged idx }} info ] );
%        [ {'tag'} {tagname} { 'SelectionChangeFcn' } {{ @callRedirect @radioSelectionChanged idx }} {'SelectedObject'} {variable} info ] );
        
    radio = zeros(1,size(radios,2));
    for i=1:size(radios,2)
        radiotextindex = findEntryIndexByName(data, radios{i} );
        if isempty(radiotextindex)
            radio(i) = uicontrol( 'Parent', group, 'style', 'radio', 'String', radios{i}, 'Position', [ 0 (i-1)*15 60 20 ],'Tag', [tagname '_' num2str(i)] );
            info2 = cell(1,0);
        else
            radio(i) = uicontrol( 'Parent', group, 'style', 'radio', 'Position', [ 0 (i-1)*15 60 20 ], 'Tag', [tagname '_' num2str(i)]);
            info2 = {'String' radios{i} };
        end
        data = storeGuiHandle(data, [tagname '_' num2str(i)], radio(i) );
        for n = 1:2:size(info,2) % we need to copy info properties to radio button properties
            try
                get(radio(i),info{1,n}); % crash if property is inexistent
                i2l = size(info2,2);
                info2{1,i2l+1} = info{1,n};
                info2{1,i2l+2} = info{1,n+1};
            catch %#ok<CTCH>
            end
        end
        if size(info2,2) > 0
            data = storeUpdateEntry( data, radio(i), info2 );
        end
    end
    
    % set SelectedObject now because the radios are not installed when we
    % call the first storeUpdateEntry
    doUpdateSelectedObject( data, group, variable, idx );
    radiogroup = [ group radio ];
end

function [ data, tabgroup ] = addTabGroup( data, tagname, fig, variable, tabs, info )
    %experimental_warning( 'uitab' );
    if data.GUILIB.matlabVersion < 2010.5
        exp = true;
    else
        exp = false;
    end
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    fig = resolveHandle( data, fig );
    checkTagnameUnique(data, tagname);     
    idx = findEntryIndexByName(data, variable );
    if isempty(idx); throwError([ 'variable ' variable ' not found']); end;
    if exp
        group = uitabgroup('v0', fig );
    else
        warning('off','MATLAB:uitabgroup:OldVersion')
        group = uitabgroup(fig);
        try
            if data.GUILIB.matlabVersion >= 2010.5
                jObject = findjobj(group);
                set(group,'UserData', jObject);
            end
        catch
        end
    end
    set_dummy_pos( fig, group );
    set(group, 'unit', 'pixels');
    data = storeGuiHandle(data, tagname, group);
    cb_prop = 'SelectionChangeCallback';
    try % the callback was renamed in R2010b
        get(group,cb_prop)
    catch %#ok<CTCH>
        % and was renamed in 2014b ...
        if data.GUILIB.matlabVersion < 2010.5
            cb_prop = 'SelectionChangeFcn';
        else
            cb_prop = 'SelectionChangedFcn';
        end
    end

    tab = zeros(1,size(tabs,2));
    for i=1:size(tabs,2)
        radiotextindex = findEntryIndexByName(data, tabs{i} );
        if isempty(radiotextindex)
            if exp
                tab(i) = uitab('v0', group, 'title',tabs{i} , 'unit', 'pixels');
            else
                tab(i) = uitab( group, 'title',tabs{i} , 'unit', 'pixels');
            end
        else
            if exp
                tab(i) = uitab('v0', group, 'unit', 'pixels');
            else
                tab(i) = uitab(group, 'unit', 'pixels');
            end
            data = storeUpdateEntry( data, tab(i), {'title' tabs{i} } );
        end
        %if ~isempty(tag); set( tab(i), 'Tag', [tag '_' num2str(i)]); end;
    end
    
    % delayed registration; if the first uitab is created it calls the
    % SelectionChangeFcn callback but the infrastructure is not completely
    % installed
    if exp
      data = storeUpdateEntry( data, group, ...
           [ {'tag'} {tagname} { cb_prop } {{ @callRedirect @tabindexChanged idx }} {'SelectedIndex'} {variable} info ] );
    else
      data = storeUpdateEntry( data, group, ...
           [ {'tag'} {tagname} { cb_prop } {{ @callRedirect @tabindexChanged2010 idx }} {'SelectedIndex'} {variable} info ] );
    end

    tag = get( group, 'Tag' );
    if ~isempty(tag); 
        for i=1:size(tabs,2)
            tagname = [tag '_' num2str(i)];
            set( tab(i), 'Tag', tagname); 
            data = storeGuiHandle(data, tagname, tab(i));
        end
    end
    
    % set SelectedObject now because the radios are not installed when we
    % call the first storeUpdateEntry
    doUpdateTabObject( data, group, variable, idx );
    tabgroup = [ group tab ];
end

function [ data,mtree] = addTree( data, tagname, fig, nodeData, nodeSelected, info )
    experimental_warning( 'uitree' );
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    fig = resolveHandle( data, fig );
    checkTagnameUnique(data, tagname);     
    n_data = findEntryIndexByName(data, nodeData );
    if isempty(n_data); throwError([ 'variable ' nodeData ' not found']); end;
    s_data = findEntryIndexByName(data, nodeSelected );
    if isempty(s_data); throwError([ 'variable ' nodeSelected ' not found']); end;
    [jtree, mtree] = uitree('v0','SelectionChangeFcn',{ @handleTreeSelection s_data } );
    if data.GUILIB.matlabVersion >= 2014.5
        set(mtree,'UserData',jtree);
    end
    set(mtree, 'Parent', fig);  % fix the uitree Parent
    set_dummy_pos( fig, mtree );
    data = storeGuiHandle(data, tagname, mtree);    
    data = storeUpdateEntry( data, mtree, [ {'tag'} {tagname} 'TreeData' nodeData info ] );
end

function [data,mtree] = addTreeDyn( data, tagname, fig, nodeData, nodeSelected, info)
    experimental_warning( 'uitree' );
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    fig = resolveHandle( data, fig );
    checkTagnameUnique(data, tagname); 
    n_data = findEntryIndexByName(data, nodeData );
    if isempty(n_data); throwError([ 'variable ' nodeData ' not found']); end;
    s_data = findEntryIndexByName(data, nodeSelected );
    if isempty(s_data); throwError([ 'variable ' nodeSelected ' not found']); end;
    data.GUILIB.treeDynParentNode = [];
    [jtree, mtree] = uitree('v0','SelectionChangeFcn',{ @handleTreeDynSelection s_data }, 'ExpandFcn', { @handleTreeDynExpand n_data} );
    set(mtree, 'Parent', fig);  % fix the uitree Parent
    if data.GUILIB.matlabVersion >= 2014.5
        set(mtree,'UserData',jtree);
    end    
    set(jtree, 'NodeCollapsedCallback', {@handleTreeDynNodeCollapsed, fig });
    set(jtree, 'NodeWillExpandCallback', { @handleTreeDynNodeWillExpand, fig } );
    set(jtree, 'NodeWillCollapseCallback', { @handleTreeDynNodeWillColapse, fig } );

    %set(jtree, 'NodeDroppedCallback', { @handleTreeDynDrop n_data } );
    set_dummy_pos( fig, mtree );
    data = storeGuiHandle(data, tagname, mtree);    
    data = storeUpdateEntry( data, mtree, [ {'tag'} {tagname} 'TreeDynData' nodeData info ] );
end

function data = addDeref(data, tagname, variable )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if ~isfield(data.GUILIB,'timerRegistry')
        data.GUILIB.timerRegistry = cell(2,0);
        data.GUILIB.timerEntries = 0;
    end
    checkTagnameUnique(data, tagname);     
    idx = findEntryIndexByName(data, variable );
    if isempty(idx); throwError([ 'variable ' variable ' not found']); end;

    timr = timer('TimerFcn',{@timerCallback, data.GUILIB.figures(1), idx},  'tag',tagname, 'BusyMode', 'drop', 'StartDelay', 0.1); %, );
    data = storeGuiHandle(data, tagname, timr);
    data.GUILIB.timerEntries = data.GUILIB.timerEntries + 1;
    data.GUILIB.timerRegistry{1,data.GUILIB.timerEntries} = timr;
    data = storeUpdateEntry( data, timr, [{'deref'} {variable}]);
end

function [ data,labelcontainer ] = addLabel( data, tagname, parent, variable, info )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    parent = resolveHandle( data, parent );
    checkTagnameUnique(data, tagname);     
    col = getBackgroundColorFromHandle( parent );
    entryIndex = findEntryIndexByName(data, variable );
    if isempty(entryIndex)
        jLabel = javaObjectEDT('javax.swing.JLabel',variable);
        [labelcomponent,labelcontainer] = javacomponent(jLabel,[100,10,140,120],parent); %#ok<ASGLU>
        data = storeUpdateEntry( data, labelcontainer, [ {'tag'} {tagname} {'Interruptible'} {'off'} {'backgroundcolor'} {col} info] );
    else
        jLabel = javaObjectEDT('javax.swing.JLabel',' ');
        [labelcomponent,labelcontainer] = javacomponent(jLabel,[100,10,140,120],parent); %#ok<ASGLU>
        data = storeUpdateEntry( data, labelcontainer, [ {'Text'}, {variable} {'tag'} {tagname} {'Interruptible'} {'off'} {'backgroundcolor'} {col} info] );
    end
    set_dummy_pos( parent, labelcontainer );
    data = storeGuiHandle(data, tagname, labelcontainer);
end


function handleTreeDynSelection (jTree, jEvent, dataIndex )
% this callback is uncommon because we dont use callRedirect
% the handle is a java object handle
    data = getRoot( jTree.UIContainer ); % get matlab container
    selectedNodes = jTree.getSelectedNodes;
    if ~isempty(selectedNodes)
        l = length(selectedNodes);
        value = cell(1,l);
        for i=1:l % get all user objects
            value{1,i} = selectedNodes(i).getUserObject();
        end
    else
        value = []; %#ok<NASGU>
    end    
    eval([ data.GUILIB.dataRegistry{1,dataIndex} ' = value;']);
    [data,redraw] = doEntryCallbackRedraw( data, struct('hObject', jTree.UIContainer, 'jObject',jTree, 'dataIndex',dataIndex, ...
        'valueString',data.GUILIB.dataRegistry{1,dataIndex},'jEvent',jEvent) );
    if isempty(redraw); data = doGuiUpdate( data ); end;
    storeUserData( data );
end

function handleTreeDynDrop(jTree, jEvent, dataIndex )
    data = getRoot( jTree.UIContainer ); % get matlab container
    [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',jTree.UIContainer ,'jObject',jTree, ...
        'dataIndex', dataIndex, 'sourceNode',jEvent.getSourceNode.getUserObject,'targetNode',jEvent.getTargetNode.getUserObject,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
    if isempty(redraw); data = doGuiUpdate( data ); end;
    storeUserData( data );    
end

function cNode = handleTreeDynNodeWillColapse(jTree, evt, figure) %#ok<INUSD,INUSL>
    % called before a node will collapse
    cNode = evt.getCurrentNode();
end

function cNode = handleTreeDynNodeWillExpand(jTree, evt, figure)
    % called before a tree will expand
    % we need to save the parent node somewhere in the data structure
    cNode = evt.getCurrentNode();
    if cNode.isRoot
        jTree.setLoaded(cNode,false);
        jTree.reloadNode(cNode);
    end
     data = getRoot(figure); % get matlab container
     data.GUILIB.treeDynParentNode = cNode;
     storeUserData( data );
end

function data = expandTreeDyn(data, htree, value)
    % expand node with (user!) value of tree with handle htree
    % called from application
    data = doGuiUpdate( data );
    htree = resolveHandle( data, htree, false );
    if data.GUILIB.matlabVersion < 2014.5
        jtree = get(htree,'FigureChild'); % get java object form matlab handle
    else
        jtree = get(htree,'UserData');
    end
    root = jtree.getRoot();
    if valueCompare(value, root.getUserObject())
        jtree.expand(root);
        drawnow;
        return;
    end
    expandSub(jtree, root, value);
end

function expandSub(jtree, root, value)
    % walk through tree search for node with (user!) value and expand it
    % called from expandTreeDyn
    for i=1:root.getChildCount
        node = root.getChildAt(i-1);
        if valueCompare(value, node.getUserObject())
            jtree.expand(node);
            drawnow;
        end
        if ~jtree.isLoaded(node); continue; end
        expandSub(jtree, node, value);
    end
end

function result = valueCompare(v1, v2)
    % compare value1 and value2
    % helper function used by expandSub and expandTreeDyn
    result = false;
    if isempty(v1)
        if ~isempty(v2); return; end
        result = true;
    elseif isnumeric(v1)
        if ~isnumeric(v2); return; end
        result = isequal(v1,v2);
    elseif ischar(v1)
        if ~ischar(v2); return; end
        result = strcmp(v1,v2);
    elseif ishandle(v1)
        if ~ishandle(v2); return; end
        result = v1==v2;
    elseif iscell(v1)
        if ~iscell(v2); return; end
        %TODO: compare cell array
    end
end

function handleTreeDynNodeCollapsed(jTree, evd, figure) %#ok<INUSD>
    %display(['' 'DATA_GUI3::handleTreeDynNodeCollapsed']);
    evdnode  = evd.getCurrentNode;
    if ~evdnode.isRoot
        jTree.setLoaded(evdnode,false);
        jTree.reloadNode(evdnode);
    else
        %display(['' 'DATA_GUI3::handleTreeDynNodeCollapsed root']);
    end
end

function nodes = handleTreeDynExpand(jTree, hObject, dataIndex)
    %display(['' 'DATA_GUI3::handleTreeDynExpand'])
    data = getRoot( hObject ); % get matlab container
    
    if ~isempty(data.GUILIB.treeDynParentNode)
        userObject = data.GUILIB.treeDynParentNode.getUserObject();
    else
        userObject = [];
    end
    
    [data,nodesList] = doEntryCallbackRedraw( data, struct('hObject',jTree.UIContainer ,'jObject',jTree, ...
        'dataIndex', dataIndex, 'valueString',data.GUILIB.dataRegistry{1,dataIndex}, 'parentNode', userObject ) );    
    if isempty(nodesList) || size(nodesList,1) == 0
        nodes = [];
    else
        for i=1:size(nodesList,1)
            nodes(i) = uitreenode('v0',jTree.UIContainer, nodesList{i,3}, nodesList{i,4}, ~nodesList{i,2}); %#ok<AGROW>
            nodes(i).setUserObject(nodesList{i,1});
        end
    end
    data.GUILIB.treeDynParentNode = [];
    storeUserData( data );
end

function error = doUpdateTreeDynRoot( data, hObject, value, dataIndex ) %#ok<INUSD>
    error = [];
    if ~ishandle(hObject); return; end
    if data.GUILIB.matlabVersion < 2014.5
        jtree = get(hObject,'FigureChild'); % get java object form matlab handle
    else
        jtree = get(hObject,'UserData');
    end
    %display(['' 'DATA_GUI3::doUpdateTreeDynRoot'])

    v = eval([value ';']);
    if isempty(v); return; end
    
    oldRoot = jtree.getRoot();
    if isempty(oldRoot) % create new root node
        root = uitreenode('v0', hObject, v{1,3}, v{1,4},v{1,2});
        root.setUserObject(v{1,1});
        jtree.setRoot(root);
        jtree.repaint();
    else %update old root node
        oldRoot.setName(v{1,3});
        oldRoot.setUserObject(v{1,1});
        l = logical(v{1,2});
        if ~l
            jtree.collapse(oldRoot);
            oldRoot.removeAllChildren();
        end
        oldRoot.setAllowsChildren(l);
        jtree.repaint();
    end
end

function error = doUpdateDragDrop( data, hObject, value, dataIndex )
    error = [];
    if ~ishandle(hObject); return; end
    jtree = getJavaFigureChild(data, hObject);
    if isempty(jtree); return; end
    %jtree = get(hObject,'FigureChild'); % get java object form matlab handle
    eval(['v=' value ';']);
    if v==true
        set(jtree, 'NodeDroppedCallback', { @handleTreeDynDrop dataIndex } );
        jtree.setMultipleSelectionEnabled(true);
    else
        jtree.setMultipleSelectionEnabled(false);
    end
end

function error = doUpdateHComponent( data, hObject, value, hcomponent_property )
    error = [];
    if ~ishandle(hObject); return; end
    try
        if data.GUILIB.matlabVersion < 2014.5
            hcomponent = get(hObject,'FigureChild'); % get java object form matlab handle
            fc = handle(get(hcomponent,'FigureComponent'));
        else
            fc = get(hObject, 'JavaPeer');
        end
        eval(['v=' value ';']);
        if ischar(v) %#ok<NODEF>
            set(fc,hcomponent_property,v);
        else
            if isnumeric(v)
                v = num2str(v);
                set(fc,hcomponent_property,v);
            else
                display('something went wrong in doUpdateHComponent(unkown data type)');
            end
        end
    catch
        display('something went wrong in doUpdateHComponent');
    end
end


function error = doUpdateDeref( data, hObject, value, dataIndex ) 
    eval(['v=' value ';']);
    error = [];
    running = get(hObject,'Running');
    if v~=0
        if strcmp(running,'off')
            set(hObject,'TimerFcn',{@timerCallback, data.GUILIB.figures(1), dataIndex }, 'BusyMode','drop','ExecutionMode','singleShot','StartDelay',0.5);
            set(hObject,'StopFcn',{@derefTimerStoped, data.GUILIB.figures(1), dataIndex, value});
            start(hObject);
        end
    else
        stop(hObject);
    end
end

function error = doUpdateTimer( data, hObject, value, dataIndex )  %#ok<INUSL,INUSD>
    eval(['v=' value ';']);
    error = [];
    running = get(hObject,'Running');
    if isnumeric(v)
        if v==0
            if strcmp(running,'on')
                stop(hObject);
            end
        elseif v==1
            if strcmp(running,'off')
                start(hObject);
            end
        end
    end
end

function error = doUpdateBackgroundColor(data, hObject, value, dataIndex) %#ok<INUSD,INUSL>
    eval(['v=' value ';']);
    set(hObject,'Backgroundcolor',v);
    error = [];
end

function error = doUpdateBackgroundColorJava(data, hObject, value, dataIndex) %#ok<INUSL>
    if ~isempty(dataIndex)
        eval(['v=' value ';']);
    else
        v = value;
    end
    if ~isnumeric(v) || length(v) ~= 3
        throwError('hobject is a java object and need numeric color information like [255 0 0]');
    end
    col = javax.swing.plaf.ColorUIResource(uint8(v(1)*255),uint8(v(2)*255),uint8(v(3)*255));
    set(hObject,'Background',col);
    error = [];
end

% private gui api callbacks

function derefTimerStoped(hObject, event, fig, dataIndex, value) %#ok<INUSL>
    % called if a deref timer stopped, we use this to restart the timer if nescessary
    data = get(fig,'userdata');
    if isempty(data)
        start(hObject);
        return; 
    end
    eval(['v=' value ';']);
    if v~=0
        start(hObject); 
    end
end

function timerCallback( obj, event, fig, dataIndex )
    try
        if ~isempty(get(0,'CallbackObject')); return; end % skip timer if we are in any callback function
        data = get(fig,'userdata');
        if isempty(data); return; end
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',obj, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex},'event',event) );
        if isempty(redraw); data = doGuiUpdate( data ); end;
        storeUserData( data );
    catch exc
        t = timerfindall();
        if ~isempty(t); 
            stop(t)
            delete(t); 
            shutdownDataGUI(exc, data.GUILIB.isDesktop);
        end
    end
end

function [data,redraw] = pushbuttonPressed( data, hObject, dataIndex )
    % called from callRedirect
    [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
end

function [data,redraw] = uicontrolValueChanged( data, hObject, dataIndex )
    % called from callRedirect
    value = get(hObject, 'value');
    [data,redraw] = updateEntry( data, dataIndex, value, true );
    if isempty(redraw)
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
    end
end

function [data,redraw] = tabindexChanged( data, hObject, dataIndex )
    % called from callRedirect
    value = get(hObject, 'SelectedIndex');
    [data,redraw] = updateEntry( data, dataIndex, value, true );
    if isempty(redraw)
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );        
    end
end

function [data,redraw] = tabindexChanged2010( data, hObject, dataIndex )
    % called from callRedirect
    child = get(hObject,'Children');
    selected = get(hObject,'SelectedTab');
    mytag = get(selected,'tag');
    tags = get(child,'tag');
    value = [];
    for i=1:length(tags)
        if strcmp(tags(i),mytag)
            value = i;
            break;
        end
    end
    [data,redraw] = updateEntry( data, dataIndex, value, true );
    if isempty(redraw)
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );                
    end
end

function [data,redraw] = tableDataChanged( data, hObject, dataIndex )
% callback from gui element, redirected from callRedirect
    value = get(hObject, 'data');
    [data,error] = updateEntry( data, dataIndex, value, true );
    if ~isempty(error)
        set( hObject, 'BackgroundColor', [ 1 0 0] );
        [data,redraw] = doErrorCallback( data, struct('error', error,'hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
    else
        col = getGuiElementDefaultColor( data, hObject );
        set( hObject, 'BackgroundColor', col );
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );                
    end
end

function [data,redraw] = edittextChanged( data, hObject, dataIndex )
% callback from gui element, redirected from callRedirect
    value = get(hObject, 'string');
    idx = findUpdateIndexByHandle( data, hObject );
    if isempty( idx ); throwError('INTERNAL ERROR: edittextChanged handle not in list. punish me'); end;
    [data,error] = updateEntry( data, dataIndex, value, true );
    if ~isempty(error)
        data.GUILIB.guiRegistry{6,idx} = error;
        set( hObject, 'BackgroundColor', [ 1 0 0] );
        [data,redraw] = doErrorCallback( data, struct('error', error,'hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
    else
        data.GUILIB.guiRegistry{6,idx} = [];
        col = data.GUILIB.guiRegistry{5,idx};
        set( hObject, 'BackgroundColor', col );
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
    end
end

function [data,redraw] = radioSelectionChanged( data, hObject, dataIndex )
    h = get( hObject, 'SelectedObject' );
    c = get( hObject, 'Children' );
    value = [];
    for i=1:size( c,1 )
        if c(i) == h
            value = i;
            break;
        end
    end
    if isempty(value)
        throwError( ' not in list' );
    end
    [data,error] = updateEntry( data, dataIndex, value, false );
    if ~isempty(error)
        [data,redraw] = doErrorCallback( data, struct('error', error,'hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
        return;
    end
    [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );    
end

function handleTreeSelection(jTree, jEvent, dataIndex ) %#ok<INUSL>
% this callback is uncommon because we dont use callRedirect
% the handle is a java object handle
    data = getRoot( jTree.UIContainer ); % get matlab container
    selectedNodes = jTree.getSelectedNodes;
    if ~isempty(selectedNodes)
        l = length(selectedNodes);
        value = zeros(1,l);
        for i=1:l % get all user objects, every node has an integer index
            value(1,i) = selectedNodes(i).getValue();
        end
    else
        value = [];
    end
    [data,error] = updateEntry( data, dataIndex, value, false );
    if isempty( error )
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject', jTree.UIContainer, 'jObject',jTree, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );    
    else
        [data,redraw] = doErrorCallback( data, struct('error', error,'hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
    end
    if isempty(redraw); data = doGuiUpdate( data ); end
    storeUserData( data );
end

function handleJavaCallback( jAxis, jEventData, hObject, callback, property)
    cbo = get(0,'CallbackObject');
    if ~isempty(cbo) % skip if we are in any other callback function
        try
            type = get(cbo,'Component');
        catch
            type = get(cbo,'Components');
            type = type(1);
        end
        if strfind(type,'com.mathworks.hg')==0; return; end;
    end 
    if ~ishandle(hObject); return; end
    
    try
        current_date = java.util.Date(); % flush old events
        current_time = current_date.getTime();
        time_diff = current_time - jEventData.getWhen;
        if time_diff > 300; return; end
    catch
    end

    data = getRoot( hObject);
    event =  struct('hObject',hObject,'jObject',jAxis, 'dataIndex',callback, ...
        'valueString','','jEvent',jEventData,'property', property, ...
        'currentAxes',[], 'isInAxes', false, 'gcaX', [], 'gcaY', [], ...
        'gcaMaxX', [], 'gcaMaxY', []);
    pl = get(0,'PointerLocation');
    wp = get(hObject,'Position');
    event.X = pl(1) - wp(1);
    event.Y = pl(2) - wp(2);
    
    all_axes = findobj(hObject, 'type', 'axes');
    if ~isempty(all_axes) % any axes?
        all_pos = get(all_axes ,'position');
        if iscell(all_pos) % cell array -> more than one axes in figure
            in_axes = cellfun(@(x) (event.X-x(1)>= 0)&&(event.Y-x(2) >= 0)&&(event.X-x(1)<x(3))&&(event.Y-x(2)<x(4)) , all_pos);
            if any(in_axes)
                indx = 1:length(in_axes);
                in_axes = indx(in_axes==1);
                in_axes = in_axes(1);
                ca_pos = all_pos{in_axes};
                event.currentAxes = all_axes(in_axes);
                event.isInAxes = 1;
                event.gcaX = event.X - ca_pos(1);
                event.gcaY = event.Y - ca_pos(2);
                event.gcaMaxX = ca_pos(3);
                event.gcaMaxY = ca_pos(4);
            end
        else % no cell only one axes
            x = all_pos;
            in_axes = (event.X-x(1)>= 0)&&(event.Y-x(2) >= 0)&&(event.X-x(1)<x(3))&&(event.Y-x(2)<x(4));
            if in_axes 
                event.currentAxes = all_axes;
                event.isInAxes = 1;
                event.gcaX = event.X - x(1);
                event.gcaY = event.Y - x(2);
                event.gcaMaxX = x(3);
                event.gcaMaxY = x(4);
            end
        end
    end
    
    [data,redraw] = doEntryCallbackRedraw( data, event);
    if isempty(redraw); data = doGuiUpdate( data ); end;
    storeUserData( data );
end

function [data,redraw] = handleTableSelected( data, hObject, callback )
    [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject,'dataIndex',callback,'valueString','','Indices',data.eventdata.Indices) );
end

function [data,redraw] = handleWindowXXX( data, hObject, callback )
    % called from callRedirect
    % used with 'WindowScrollWheelFcn' 'WindowButtonUpFcn' 'WindowButtonMotionFcn'
    pl = get(0,'PointerLocation');
    wp = get(hObject,'Position');
    event =  struct('hObject',hObject,'dataIndex',callback,'valueString','') ;
    event.X = pl(1) - wp(1);
    event.Y = pl(2) - wp(2);
    if isfield(data.eventdata,'VerticalScrollCount')
        event.VerticalScrollCount = data.eventdata.VerticalScrollCount;
        event.VerticalScrollAmount = data.eventdata.VerticalScrollAmount;
    end
    [data,redraw] = doEntryCallbackRedraw( data, event);
end

function callRedirect( hObject, eventdata, callfunction, parameter1, parameter2 )
% general callback function with data structure support
% this function will only be called directly from the gui element and
% refers only to internal callback functions (EXCEPT: handleJAVACallbacks)
    data = getRoot( hObject );
    data.eventdata = eventdata;
    if nargin == 3
        [data,redraw] = callfunction(data, hObject); % internal callback
    elseif nargin == 4 
        [data,redraw] = callfunction( data, hObject, parameter1 ); % internal callback
    elseif nargin == 5
        [data,redraw] = callfunction( data, hObject, parameter1, parameter2 ); % internal callback
    end
    if isempty(redraw); data = doGuiUpdate( data ); end;
    storeUserData( data );
end

function wrapped = callbackWrapper(matlabCallback, param1, param2)
%[data redraw] = doEntryCallbackRedraw( data, struct('hObject',hObject, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );
    switch nargin
        case 1
            wrapped = {@callbackWrapperLander matlabCallback};
        case 2
            wrapped = {@callbackWrapperLander matlabCallback param1};
        case 3
            wrapped = {@callbackWrapperLander matlabCallback param1 param2};
        otherwise
            throwError('wrapCallback need 1 to 3 parameter(s)');
    end
end

function callbackWrapperLander(hObject, eventdata, matlabCallback, param1, param2)
    data = getRoot( hObject );
    data.eventdata = eventdata;
    event = struct('hObject',hObject, 'valueString',[],'dataIndex', matlabCallback);
    if nargin == 4 
        event.parameter1 = param1;
    elseif nargin == 5
        event.parameter2 = param2;
    end
    [data,redraw] = doEntryCallbackRedraw( data,  event);
    if isempty(redraw); data = doGuiUpdate( data ); end;
    storeUserData( data );
end

function data = closeFigure( data, hObject )
    if nargout==0; throwError( 'do not revert return data structure' ); end;

    figurecnt = size(data.GUILIB.figures,2); % search object in figures list

    if figurecnt<2
        display('not child figure');
        return;
    end
    hObject = resolveHandle( data, hObject );

    idx = [];
    for i=1:figurecnt
        if hObject == data.GUILIB.figures(i)
            idx = i;
            break;
        end
    end

    if isempty(idx)
        display('figure not found');
        return
    end
    
    data.GUILIB.figures = [ data.GUILIB.figures(:,1:idx-1) data.GUILIB.figures(:,idx+1:figurecnt) ];
    
    c = getAllChildren( hObject );
    for i=1:size(c,1)
        data = removeUpdateEntry( data, c(i) );
    end
    data = removeUpdateEntry( data, hObject ); % remove figure from update list
    data = cleanupAxesInset(data, hObject);

    set( hObject, 'CloseRequestFcn', [] );
    delete(hObject); % kill figure
end

function closeFigureCallback( hObject, eventdata ) %#ok<INUSD>
% close figure callback
    try
        data = getRoot(hObject);

        for cl = 1:size(data.GUILIB.closeCallbacks,2)
            if (hObject == data.GUILIB.closeCallbacks{1,cl})
                value = data.GUILIB.closeCallbacks{2,cl};
                try
                    [data,redraw,doquit] = value(data, [], true); % user callback
                    if isempty(redraw); data = doGuiUpdate( data ); end;
                    if  doquit == false
                        storeUserData( data );
                        return
                    end
                catch exp
                    txt = ['exception in ' exp.stack(1).name ...
                        ' line ' num2str(exp.stack(1).line) ' ' exp.message ];
                    display([txt '']);
                end
                break;
            end
        end

        if hObject == data.GUILIB.figures(1) % main (first) figure
            if isfield(data.GUILIB,'timerRegistry') % kill timer
                for i=1:data.GUILIB.timerEntries
                    timr = data.GUILIB.timerRegistry{1,i};
                    if isvalid(timr)
                        stop(timr);
                        delete(timr);
                    end
                end
            end
            delete(data.GUILIB.figures); % close figures
        else
            %dumpGuiRegistry(data);
            figurecnt = size(data.GUILIB.figures,2); % search object in figures list
            for i=1:figurecnt
                if hObject == data.GUILIB.figures(i)
                    idx = i;
                    break;
                end
            end
            if ~isempty(idx) % remove from list
                data.GUILIB.figures = [ data.GUILIB.figures(:,1:idx-1) data.GUILIB.figures(:,idx+1:figurecnt) ];
            end
            c = getAllChildren( hObject );
            for i=1:size(c,1)
                data = removeUpdateEntry( data, c(i) );
            end
            data = removeUpdateEntry( data, hObject ); % remove figure from update list
            data = cleanupAxesInset(data, hObject);

            delete(hObject); % kill figure
            storeUserData(data); % store data
        end
    catch exc
        display(['closeFigureCallback crashed: ' exc.message]);
        delete(hObject);
    end
end

function out = cellShift( in, step )
    l = size(in, 2);
    if step >= l; out = []; return; end;
    out = in(1,step+1:l);
end

function checkTagnameUnique(data, tagname)
% check if tagname is in use and throw an exception if it is
    if isempty(tagname)
        throwError('tagname is empty');
    end
    if ~ischar(tagname)
        throwError('tagname is not a string');
    end
    if ~isempty(resolveHandle( data, tagname, true ))
        throwError(['tagname ' tagname ' is allready in use']);
    end
    for i=1:length(tagname)
        t = tagname(i);
        if t>='a' && t<='z'; continue; end
        if t>='A' && t<='Z'; continue; end
        if i==1; throwError(['tagname (' tagname ') must start with a letter' ]); end
        if t>='0' && t<='9'; continue; end
        if t=='_'; continue; end
        throwError(['illegal character in tagname ('  tagname ') : ''' t '''']);
    end
    if length(tagname) > namelengthmax
        throwError(['tagname ('  tagname ') is longer than max of ' num2str(namelengthmax) ]);
    end
end

function data = storeGuiHandle(data, tagname, hObject)
    if ~isfield(data, 'handles') || ~isstruct(data.handles)
        data.handles = struct();
    end
    data.handles.(tagname) = hObject;
end

function hObject = resolveHandle( data, hObject, allowEmptyResult )
 % find handle if hObject is a tag-string, if hObject is a handle return it
 % TODO: merge with getAllChildren
    if ishandle(hObject); return; end;
    if ~ischar(hObject); throwError('resolveHandle need handle or tag-string');end;
        
    h2 = [];
    for i=1:length(data.GUILIB.figures)
        h = getAllChildren( data.GUILIB.figures(i) );
        h2 = cat(1,h2, h);
    end
    h = cat(1,h,data.GUILIB.figures');        
    
    oldObject = hObject;
    hObject = unique(findobj(h,'Tag',hObject));
    if isempty(hObject) && isfield(data.GUILIB,'timerRegistry') % search timer
        for i=1:data.GUILIB.timerEntries
            tag = get(data.GUILIB.timerRegistry{1,i}, 'tag' );
            if strcmp(tag, oldObject)
                hObject = data.GUILIB.timerRegistry{1,i};
                break;
            end
        end
    end
    if nargin==3 && allowEmptyResult==true
        return
    else
        if isempty(hObject); throwError(['resolveHandle: hObject ' oldObject ' not found']); end;
    end
    if size(hObject,1) ~= 1; throwError(['resolveHandle: hObject ' oldObject ' more than once']);	end;
end

function children = getAllChildren( hObject )
    children = [];
    c = get( hObject, 'Children' );
    if isempty(c); return; end;
    for i=1:size(c,1)
        %children = [ children getAllChildren( c(i) ) ]; %#ok<AGROW>
        children = cat(1,getAllChildren( c(i) ), children);
    end
    children = cat(1,c, children);
end

function set_dummy_pos( fig, hObject )
    % todo : remove me
    pos = [ 0 0 0.1 0.1 ];
    c = size( get( fig, 'Children' ), 1 );
    c = 100 - c;
    pos(2) = fix( c / 10 );
    pos(1) = ((c - (pos(2) * 10)) / 10);
    pos(2) = pos(2) / 10;
    set( hObject, 'Units', 'normalized', 'Position', pos, 'visible', 'off' );
end

% private gui api
function data = getRoot( hObject )
    type = get( hObject, 'type' );
    while ~strcmp( type, 'figure' )
        hObject = get( hObject, 'parent' );
        type = get( hObject, 'type' );
    end
    data = get( hObject, 'UserData' );
end

function col = getBackgroundColorFromHandle( hObject )
    try
        type = get( hObject, 'type' );
        switch type
            case {'uipanel','uicontrol','uitable'}
                col = get( hObject, 'BackgroundColor' );
            case 'figure'
                col = get( hObject, 'Color' );
            otherwise
                throwError(['need support for ' type ' in data_gui3::getBackgroundColorFromHandle']);
                col = [];
        end
    catch
        try
            col = hObject.getBackground();
            col = [col.getRed() col.getGreen() col.getBlue()];
            col = col / 255;
        catch e
            m = e.message;
            display(['crash in datagui3:getBackgroundColorFromHandle ' m]);
            col = [];
        end

    end
end

function storeUserData( data )
% store data structure in userdata field of main figure
% this is typicaly the last call in a callback function
    try 
        if ~ishandle(data.GUILIB.figures(1)); return; end % main figure closed
        set( data.GUILIB.figures, 'UserData', data );
    catch exp
        display('storeUserData crashed - your data structure is lost!')
        display(exp.message);
    end
end

function data = doGuiUpdate( data )
    for i=1:size(data.GUILIB.guiRegistry,2);
        if ~isempty(data.GUILIB.guiRegistry{6,i}); 
            data.GUILIB.guiRegistry{6,i} = [];
            continue
        end % object has wrong entries, no more update
        hObject = data.GUILIB.guiRegistry{1,i};
        maintained = data.GUILIB.guiRegistry{2,i};
        if ~isempty(maintained)
            s = dataToStruct(data, maintained);
            set( hObject, s ); 
        end
        onetime = data.GUILIB.guiRegistry{3,i};
        if ~isempty(onetime)
            prop = onetime(1,:);
            valu = onetime(2,:);
            set( hObject, prop, valu );
            data.GUILIB.guiRegistry{3,i} = [];
        end
        special = data.GUILIB.guiRegistry{4,i};
        if ~isempty( special )
            for n=1:size(special,2)
                c = special{1,n};
                if ~isempty( c{1}(data, c{2} , c{3}, c{4} ) )
                    break; % something went wrong
                end
            end
        end
    end
    data = checkAndDoResize(data);
end

function error = doUpdateListboxObject( data, hObject, value, dataIndex )
    eval(['v=' value ';']);
    v = fix(v); %#ok<NODEF>
    [data,v,error] = checkEntry( data, dataIndex, v, true );
    if isempty(error)
        col = getGuiElementDefaultColor( data, hObject );
        set( hObject, 'value', v, 'BackgroundColor', col );
    else
        set( hObject, 'value', 1, 'BackgroundColor', [ 1 0 0] );
    end
end

function error = doUpdateSliderObject( data, hObject, value, dataIndex )
    eval(['v=' value ';']);
    [data,v,error] = checkEntry( data, dataIndex, v, true ); %#ok<NODEF>
    if isempty(error)
        col = getGuiElementDefaultColor( data, hObject );
        set( hObject, 'value', v, 'BackgroundColor', col );
    else
        constrain = getConstrain(data, dataIndex);
        default = 1;
        if isnumeric(constrain); default = constrain(1); end;
        set( hObject, 'value', default, 'BackgroundColor', [ 1 0 0] );
    end
end

function error = doUpdateSelectedObject( data, hObject, value, dataIndex )  %#ok<INUSD>
    error = [];
    eval(['v=' value ';']);
    v = fix(v); %#ok<NODEF>
    c = get( hObject, 'Children');
    if v<1 || v>size(c,1)
        set( hObject, 'BackgroundColor', [1 0 0], 'SelectedObject', [] );
        set( c, 'BackgroundColor', [1 0 0] );
        error = 'radio expects other values';
        return;
    end
    col = getGuiElementDefaultColor( data, hObject );
    set( hObject, 'SelectedObject', c(v), 'BackgroundColor', col );
    set( c, 'BackgroundColor', col );
end

function error = doUpdateEditTextObject( data, hObject, value, dataIndex )
    eval(['v=' value ';']);
    [data,v,error] = checkEntry( data, dataIndex, v, true ); %#ok<NODEF>
    if isempty(error)
        idx = findUpdateIndexByHandle( data, hObject );
        if isempty( idx ); throwError(['default color from ' value ' not found']); end;
        col = data.GUILIB.guiRegistry{5,idx};
        data.GUILIB.guiRegistry{6,idx} = [];
        set( hObject, 'string', v, 'BackgroundColor', col );
    else
        set( hObject, 'string', v, 'BackgroundColor', [ 1 0 0] );
    end
end

function error = doUpdateTableObject( data, hObject, value, dataIndex )
    eval(['v=' value ';']);
    [data,v,error] = checkEntry( data, dataIndex, v, true ); %#ok<NODEF>
    if isempty(error)
        col = getGuiElementDefaultColor( data, hObject );
        set( hObject, 'data', v, 'BackgroundColor', col );
    else
        set( hObject, 'data', v, 'BackgroundColor', [ 1 0 0] );
    end
end

function lock = table_viewline_lock(try_lock)
    persistent viewline_lock;
    lock = [];    
    
    if try_lock == true
        % get lock
        if isempty(viewline_lock)
            % lock it!
            viewline_lock = true;
            lock = true;
        else
            % allready locked
            return
        end
    else
        % release lock
        if isempty(viewline_lock)
            display(char('table_viewline_lock error: release unlocked variable'));
        else
            viewline_lock = [];
        end
    end 
end

function error = doUpdateTableViewLine( data, hObject, value, jscrollpane ) %#ok<INUSL>
    if isempty(jscrollpane)
        error = [];
        return
    end
    %if ~isempty(strfind(get(0,'CallbackObject'), 'UIScrollPane'))
    %    error = [];
    %    return
    %end
    
    if isempty(table_viewline_lock(true)) ; error = []; return ; end
    
    eval(['v=' value ';']);
    new_row = v-1;
    max_row = size(get(hObject,'Data'),1);
    
    try
        jtable = jscrollpane.getViewport.getView;
        jtable.getParent().getViewRect;
        fullheight = jtable.getParent().getViewSize.getHeight;
        single_height = fullheight / max_row;
        new_height = single_height * new_row;
        if new_height > fullheight
            new_height = fullheight;
        end
        vp = jtable.getParent().getViewPosition;
        vp.y = new_height;
        
            %old = get(jscrollpane,'AdjustmentValueChangedCallback');
            %changed = old;
            %changed{3} = [];
            %set(jscrollpane,'AdjustmentValueChangedCallback',changed);
        jtable.getParent().setViewPosition(vp);
            %set(jscrollpane,'AdjustmentValueChangedCallback',old);

    catch 
        display('doUpdateTableViewLine crashed')
    end
    table_viewline_lock(false);
    error = [];
end

function TableAdjustmentValueChangedCallback(~, event, uitable_handle, dataIndx)
    if isempty(dataIndx); return; end    
    try        
        if event.getSource().getOrientation() ~= 1
            return
        end
        if isempty(table_viewline_lock(true)) ; return ; end
        
        data = getRoot(uitable_handle);
        fullheight = event.getSource.getParent.getViewport.getView.getHeight;
        currenty = event.getValue();
        tableheight = size(get(uitable_handle,'Data'),1);
        visible_line = fix(currenty / (fullheight/tableheight)) + 1;
        
        [data,~] = updateEntry( data, dataIndx, visible_line, true );
        [data,redraw] = doEntryCallbackRedraw( data, struct('hObject',uitable_handle, 'dataIndex',dataIndx,'valueString',data.GUILIB.dataRegistry{1,dataIndx}) );

        
        %[data,redraw] = updateEntry( data, dataIndx, visible_line, true );
        
        if isempty(redraw)
            %old = get(hObject,'AdjustmentValueChangedCallback');
            %changed = old;
            %changed{3} = [];
            %set(hObject,'AdjustmentValueChangedCallback',changed);
            data = doGuiUpdate( data ); 
            %set(hObject,'AdjustmentValueChangedCallback',old);
        end;
        
        storeUserData( data );        
        
    catch 
    end
    table_viewline_lock(false);
end


function error = doUpdateTabObject( data, hObject, value, dataIndex )
    eval(['v=' value ';']);
    [data,v,error] = checkEntry( data, dataIndex, v, true ); %#ok<NODEF>
    if isempty(error)
        col = getGuiElementDefaultColor( data, hObject );
        tabs = get(hObject,'Children');
        if isempty(tabs)
            v = 0;
        elseif length(tabs) < v
            v = 0;
        end
        if data.GUILIB.matlabVersion < 2014.5
            set( hObject, 'SelectedIndex', v, 'BackgroundColor', col );
        else
            c = get(hObject,'Children');
            v = round(v);
            set(hObject,'SelectedTab',c(v));
            j = get(hObject,'UserData');
            if ~isempty(j) && length(j) >= 3
                col = javax.swing.plaf.ColorUIResource(uint8(col(1)*255),uint8(col(2)*255),uint8(col(3)*255));
                j(3).setBackground(col);
            end
        end
    else
        col = [1 0 0];
        if data.GUILIB.matlabVersion < 2014.5
            set( hObject, 'SelectedIndex', 0, 'BackgroundColor', col );
        else
            j = get(hObject,'UserData');
            if ~isempty(j) && length(j) >= 3
                col = javax.swing.plaf.ColorUIResource(uint8(col(1)*255),uint8(col(2)*255),uint8(col(3)*255));
                j(3).setBackground(col);
            end
        end
    end
end

function child = getJavaFigureChild(data, hObject)
    if data.GUILIB.matlabVersion < 2014.5
        child = get(hObject,'FigureChild'); % get java object form matlab handle
    else
        child = get(hObject,'UserData');
    end
end

function error = doUpdateTreeData( data, hObject, value, dataIndex ) %#ok<INUSD>
    % updates the entries of an uitree
    error = [];
    if data.GUILIB.matlabVersion < 2014.5
        java_objects = get(hObject,'FigureChild'); % get java object form matlab handle
    else
        java_objects = get(hObject,'UserData');
    end
    if isempty(java_objects); return; end
    v = [0 1]; % fix strange warnings in old matlab version
    eval(['v=' value ';']);
    len = size(v,1);
    if len<1; return; end
    level = cell2mat(v(:,1)); % convert level list from cell to double array
    parent_levels = level(1);
    if length(level(level==parent_levels)) ~= 1
        display('only one entry as root level allowed');
        return
    end
    
    selectedNodes = java_objects.getSelectedNodes(); % create list of selected nodes
    l = length(selectedNodes);
    selected = zeros(1,l);
    for i=1:l % get all user objects, every node has an integer index
        selected(1,i) = selectedNodes(i).getValue();
    end

    if checkTreeUpdate(v, java_objects) % check if hierarchy changed, and UPDATE text if not
        java_objects.repaint(); % manual repaint without text length adjust
        return;
    end
    list = list2Tree( v, java_objects ); % create new tree nodes
    if l > 0 % restore selected nodes
        h = cell2mat(list(:,4));
        if selected > length(h)
            return
        end
        h = h(selected);
        if ~isempty(h)
            if size(h,1)==1
                java_objects.setSelectedNode(h); 
            else
                java_objects.setSelectedNodes(h); 
            end
        end
    end
    java_objects.repaint();
end

function equal = checkTreeUpdate( list, tree )
% check if tree hierarchy must be changed, update text if not
    equal = false;
    nodes = createTreeNodeList( [], tree.getRoot() );
    nodes_len = size(nodes,1);
    if nodes_len+1 ~= size(list,1); return; end
    levels = zeros(nodes_len+1,1);
    for i=1:nodes_len
        levels(i+1) = nodes(i).getLevel();
    end
    indx = 1:1:nodes_len+1;
    olevel = cell2mat(list(:,1));
    newlevel = 0;
    for i=min(olevel):max(olevel) % normalize
        mov = indx(olevel==i);
        if ~isempty(mov)
            olevel(mov) = newlevel;
            newlevel = newlevel + 1;
        end
    end
    equal = olevel == levels;
    equal = min(equal) == max(equal);
    if equal == false; return; end
    if isempty(tree.getRoot); equal = false; return; end
    tree.getRoot.setName(list(1,2));
    for i=2:nodes_len+1
        nodes(i-1).setName(list(i,2));
    end
end

function list = createTreeNodeList( list, node )
% returns all sub nodes of a node
    if isempty(node); return; end
    for i=1:node.getChildCount()
        list = [ list node.getChildAt(i-1) ]; %#ok<AGROW>
        list = createTreeNodeList( list, node.getChildAt(i-1) );
    end
end

function list = list2Tree( list, tree )
% create uitreenode list and add it to uitree object
% input list format: { { level(int) name(string) } { ... } }
    max_l = size(list,1);
    indx = 1:1:max_l;
    for l=1:max_l
        current = list{l,1}; % current level
        has_no_child = true;
        if l < max_l && list{l+1,1} > current % check if current node has children
            has_no_child = false;
        end
        icon = list{l,3}; 
        if ~ischar(icon); icon = []; end
        list{l,4} = uitreenode('v0', l, list{l,2}, icon, has_no_child);
        list{l,4}.setUserObject(l);
        list{l,5} = has_no_child;
        levl = cell2mat(list(1:l,1)); % search parent java object
        p_n = indx(levl == current -1 ); % index list of all objects with level - 1
        if ~isempty(p_n)
            p_n = max(p_n); % the last entry from list is the parent
            list{p_n,4}.add( list{l,4} ); % add current to parent object
        end
        list{l,6} = p_n; % store parent index
    end
    if max_l > 0 
        tree.setRoot(list{1,4}); % set root node
        expd = cell2mat(list(1:l,5)); % non cell array of has_no_child
        expdh = cell2mat(list(indx(expd == 0 ),4)); % java node list of entries with children
        for l=1:length(expdh)
            tree.expand(expdh(l)); % expand nodes
        end
    end
end

function error = doUpdateTreeMultiSelect( data, hObject, value, dataIndex ) %#ok<INUSD>
    % change the multi select behaviour of a uitree
    error = [];
    if ~ishandle(hObject); return; end
    if data.GUILIB.matlabVersion < 2014.5
        java_objects = get(hObject,'FigureChild'); % get java object form matlab handle
    else
        java_objects = get(hObject,'UserData'); % get java object form matlab handle
    end
    if ~isempty(java_objects)
        if ischar(value)
            eval(['v=' value ';']);
        else
            v = value;
        end
        java_objects(1).setMultipleSelectionEnabled(v);
    end
end

function error = doUpdateAxesCallback( data, hObject, callback, unused ) %#ok<INUSD,INUSL>
    error = [];
    set(hObject,'buttondownfcn', { @callRedirect, @handleAxesButtonDown, callback, hObject });
    c = get(hObject,'children');
    if ~isempty(c)
        set(c,'buttondownfcn', { @callRedirect, @handleAxesButtonDown, callback, hObject });
    end
end

function [data,redraw] = handleAxesButtonDown( data, hObject, callback, axesHandle)
    parentFigure = get(hObject,'parent');
    while ~strcmp(get(parentFigure,'type'),'figure')
        parentFigure = get(parentFigure,'parent');
    end
    event = struct('hObject',axesHandle,'dataIndex',callback,'valueString','','axesChild',hObject);
    pl = get(0,'PointerLocation');
    wp = get(parentFigure,'Position');
    ap = get(axesHandle,'Position');
    event.FigureX = pl(1) - wp(1);
    event.FigureY = pl(2) - wp(2);
    event.X = event.FigureX - ap(1);
    event.Y = event.FigureY - ap(2);
    [data,redraw] = doEntryCallbackRedraw( data,  event);
end

function setFigureJavaFrame(hObject, property, callback)
    try
        warning off MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame
        javaframe = get(hObject, 'JavaFrame');
        axisComponent = javaframe.getAxisComponent;
        haxisComponent = handle(axisComponent, 'CallbackProperties');
        set(haxisComponent, property,{@handleJavaCallback,hObject,callback,property})
    catch exp
        txt = ['exception in ' exp.stack(1).name ...
            ' line ' num2str(exp.stack(1).line) ' ' exp.message ];
        display([txt '']);
    end
end

function setFigureJavaFrame2014b(hObject, property, callback)
    try
        warning off MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame
        javaframe = get(hObject, 'JavaFrame');
        jAxis = javaframe.getAxisComponent;
        component0 = jAxis.getComponent(0);
        %component0.getMouseListeners
        set(component0,property,{@handleJavaCallback,hObject,callback,property});
        %component0.getMouseListeners
    catch exp
        txt = ['exception in ' exp.stack(1).name ...
            ' line ' num2str(exp.stack(1).line) ' ' exp.message ];
        display([txt '']);
    end
end

function error = doUpdateSliderContinueouse(data, hObject, value, dataIndex)
% update callback of hidden and undocumented jSlider object
% we need the findjobj function written by Yair Altman
% the jslider object will be cached in the userdata property if possible
% the cache function will crash if somebody saves a different and vaild
% matlab handle inside the userdata property
    if ischar(value)
        eval(['v=' value ';']);
        [data,v,error] = checkEntry( data, dataIndex, v, true ); %#ok<ASGLU,NODEF>
    else
        if isnumeric(value)
            error = [];
            v = value;
        else
            error = 'doUpdateSliderContinueouse need numeric or ''data.xxx'' parameter';
        end
    end
    if isempty(error)
        ud  = get(hObject, 'UserData');
        if hObject==ud; return; end % avoid restart of this function if something fails
        if isempty(ud) || ~ishandle(ud)
            vis = get(hObject, 'Visible');
            if strcmp(vis,'off')
                set(hObject,'Visible','on');
                pause(0.1);
                drawnow;
            end
            try
                jSlider = findjobj(hObject);
                if isempty(jSlider); jSlider=hObject; end
            catch exp %#ok<NASGU>
                jSlider = hObject; % avoid restart of this function
            end
        else
            jSlider = ud;
        end
        if ~isempty(jSlider)
            if jSlider ~= hObject % we abuse this to prevent a restart if the function fails
                try
                    switch v
                        case 0
                            jSlider.AdjustmentValueChangedCallback = [];
                        case 1
                            callback = get(hObject, 'callback');
                            jSlider.AdjustmentValueChangedCallback = {@handleContinuouseSlider hObject callback{3}};
                    end
                catch exp %#ok<NASGU>
                    jSlider = hObject;
                end
            end
        end
        if isempty(ud)
            set(hObject, 'UserData', jSlider);
        end
    end
end

function handleContinuouseSlider( jAxis, jEventData, hObject, dataIndex) %#ok<INUSL>
    callRedirect( hObject, jEventData, @uicontrolValueChanged, dataIndex);
end

function experimental_warning( name )
    if isempty(whos('global','datagui_experimental'))
        global datagui_experimental %#ok<TLEV>
        datagui_experimental = 1;
        warning('normal:warning',[name ' support is an undocumented matlab feature'])
    end
end

function ver = getMatlabVersion()
    persistent cached_version
    if ~isempty(cached_version)
        ver = cached_version;
        return
    end    
    ver = str2double(version('-release'));
    if isnan(ver)
        v = version('-release');
        ver = str2double(v(1:4));
        if isnan(ver); throwError(['can''t resolve version string: ' v]); end
        if v(5)=='b'; ver = ver + 0.5; end
    end
    cached_version = ver;
end

function moveFigureToFront(figureHandle)
    try
        if isunix()
            unity = getenv('XDG_CURRENT_DESKTOP');
            if ~isempty(unity)
                if strcmp(unity, 'Unity')
                    return
                end
            end
            return;
        end
        pause(0.1);
        jFigPeer = get(handle(figureHandle),'JavaFrame');
        try
            jWindow = jFigPeer.fFigureClient.getWindow; %R2009
        catch exp %#ok<NASGU>
            jWindow = jFigPeer.fHG1Client.getWindow; % R2012 
        end
        jWindow.toFront();
        jWindow.requestFocus();
        jWindow.setAlwaysOnTop(true);
        try 
            pinfo = java.awt.MouseInfo.getPointerInfo(); % fix crash in R2007
            oldMouseLocation = pinfo.getLocation();
            robot = java.awt.Robot();
            robot.mouseMove(jWindow.getX() + 100, jWindow.getY() + 5);
            robot.mousePress  (java.awt.event.InputEvent.BUTTON1_MASK);
            robot.mouseRelease(java.awt.event.InputEvent.BUTTON1_MASK);
            robot.mouseMove(oldMouseLocation.getX(), oldMouseLocation.getY());
        catch exp
            txt = ['exception in ' exp.stack(1).name ...
                ' line ' num2str(exp.stack(1).line) ' ' exp.message ];
            display([txt '']);
        end
        jWindow.setAlwaysOnTop(false);
    catch exp
        txt = ['exception in ' exp.stack(1).name ...
            ' line ' num2str(exp.stack(1).line) ' ' exp.message ];
        display([txt '']);
    end
end

function data = checkAndDoResize(data)
    % called from doGuiUpdate
    % resize the figure if necessary
    resizeFigures = [];
    f = fieldnames(data.GUILIB.axeshandles);
    for i=1:size(f,1)
        s = data.GUILIB.axeshandles.(f{i,1});
        inset = get(s.handle, 'TightInset');
        if inset(1)~=s.inset(1) || inset(2)~=s.inset(2) || ...
                inset(3)~=s.inset(3) || inset(4)~=s.inset(4)
            resizeFigures = [resizeFigures s.figure]; %#ok<AGROW>
        end
    end
    
    % check and update variables who were used in the layout:
    ll = data.GUILIB.layoutData;
    for i=1:size(ll,1)
        lst = ll{i,2};
        for n=1:size(lst,1)
            variable = lst{n,1};
            value = eval([variable ';']);
            if value ~= lst{n,2}
                lst{n,2} = value;
                resizeFigures = [resizeFigures ll{i,1}]; %#ok<AGROW>
            end
        end
        ll{i,2} = lst;
    end
    data.GUILIB.layoutData = ll;
    
    resizeFigures =  unique(resizeFigures);
    for i=1:length(resizeFigures)
        data = manualResize( data, resizeFigures(i) );
    end
end

function data = updateAxesInset(data)
    % called from figureResize2 and manualResize
    % update stored inset variables, in the next doGuiUpdate we can 
    % detect if we need to resize the figure
    f = fieldnames(data.GUILIB.axeshandles);
    for i=1:size(f,1)
        s = data.GUILIB.axeshandles.(f{i,1});
        s.inset = get(s.handle, 'TightInset');
        data.GUILIB.axeshandles.(f{i,1}) = s;
    end
end

function data = updateFigurePositionDataEntry(data)
    % update data entry of position property
    for fc=1:size(data.GUILIB.figures,2)
        if data.GUILIB.figPosEntry(fc) ~= 0
            dataIndex = data.GUILIB.figPosEntry(fc);
            value = get(data.GUILIB.figures(fc),'position'); %#ok<NASGU>
            eval([data.GUILIB.dataRegistry{1,dataIndex} '= value;']); % set new value
        end
    end
end

function data = cleanupAxesInset(data, removedHandle)
    % called from closeFigure callback
    % check and update variables who were used in the layout:
    ll = data.GUILIB.layoutData;
    for i=1:size(ll,1)
        if removedHandle == ll{i,1}
            if i==1
                ll = ll(2,:);
            else
                if i==size(ll,1)
                    ll = ll(1:i-1,:);
                else
                    ll = [ ll(1:i-1,:) ; ll(i+1,:) ];
                end
            end
            break;
        end
    end
    data.GUILIB.layoutData = ll;
    
    f = fieldnames(data.GUILIB.axeshandles);
    for i=1:size(f,1)
        s = data.GUILIB.axeshandles.(f{i,1});
        if s.figure == removedHandle
            data.GUILIB.axeshandles = rmfield(data.GUILIB.axeshandles,f{i,1});
        end
    end
end

function data = addLayoutDataEntry(data, variable, figure)
    % called from registerDependendLayoutVariables
    % fill data.GUILIB.layoutData watch list
    ll = data.GUILIB.layoutData;
    indx = [];
    for i=1:size(ll,1)
        if ll{i,1} == figure
            indx = i;
            break;
        end
    end
    if isempty(indx)
        new_len = size(ll,1)+1;
        ll{new_len,1} = figure;
        value = eval([variable ';']);
        ll{new_len,2} = { variable value};
    else
        lst = ll{indx,2};
        for i=1:size(lst,1)
            if strcmp(lst{i,1},variable)
                return;
            end
        end
        lst{size(lst,1)+1,1} = variable;
        value = eval([variable ';']);
        lst{size(lst,1),2} = value;
        ll{indx,2} = lst;
    end
    data.GUILIB.layoutData = ll;
end

function data = registerDependendLayoutVariables(data, figure, layout_in)
    % search 'data.xxx' in layout definition and remark them in the watchlist
    % the watchlist will be parsed in doGuiUpdate and call the figure
    % resize function if the 'data.xxx' value has changed
    % called from addLayout
    [l,r] = size(layout_in);
    for i=1:l
        for n=1:r
            c = layout_in{i,n};
            if isempty(c); continue; end
            if iscell(c)
                data = registerDependendLayoutVariables(data, figure, c);
            else
                if ischar(c)
                    if ~isempty(regexp(c,'^data.','ONCE'))
                        data = addLayoutDataEntry(data, c, figure);
                    end
                end
            end
        end
    end
end

function s = dataToStruct(data, in) %#ok<INUSL>
    s = struct();
    c = size(in,2);
    switch c
        case 1
            eval(['s.' in{1,1} '=' in{2,1} ';']);
        case 2
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';' ]);
        case 3
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';s.' in{1,3} '=' in{2,3} ';' ]);
        case 4
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';s.' in{1,3} '=' in{2,3} ';s.' ...
                in{1,4} '=' in{2,4} ';' ]);
        case 5
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';s.' in{1,3} '=' in{2,3} ';s.' ...
                in{1,4} '=' in{2,4} ';s.' in{1,5} '=' in{2,5} ';']);
        case 6
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';s.' in{1,3} '=' in{2,3} ';s.' ...
                in{1,4} '=' in{2,4} ';s.' in{1,5} '=' in{2,5} ';s.' in{1,6} '=' in{2,6} ';']);
        case 7
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';s.' in{1,3} '=' in{2,3} ';s.' ...
                in{1,4} '=' in{2,4} ';s.' in{1,5} '=' in{2,5} ';s.' in{1,6} '=' in{2,6} ';s.' ... 
                in{1,7} '=' in{2,7} ';']);
        case 8
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';s.' in{1,3} '=' in{2,3} ';s.' ...
                in{1,4} '=' in{2,4} ';s.' in{1,5} '=' in{2,5} ';s.' in{1,6} '=' in{2,6} ';s.' ... 
                in{1,7} '=' in{2,7} ';s.' in{1,8} '=' in{2,8} ';']);
        case 9
            eval(['s.' in{1,1} '=' in{2,1} ';s.' in{1,2} '=' in{2,2} ';s.' in{1,3} '=' in{2,3} ';s.' ...
                in{1,4} '=' in{2,4} ';s.' in{1,5} '=' in{2,5} ';s.' in{1,6} '=' in{2,6} ';s.' ... 
                in{1,7} '=' in{2,7} ';s.' in{1,8} '=' in{2,8} ';s.' in{1,9} '=' in{2,9} ';']);
        otherwise
    end
end

function col = getGuiElementDefaultColor( data, hObject )
    idx = findUpdateIndexByHandle( data, hObject );
    if isempty( idx ); throwError(['default color from ' value ' not found']); end;
    col = data.GUILIB.guiRegistry{5,idx};
    if isempty(col)
        col = [ .3 .3 .3];
    end
end

function data = removeUpdateEntry( data, hObject )
    idx = findUpdateIndexByHandle( data, hObject );
    if isempty( idx ); return; end;
    data.GUILIB.guiRegistry = [ data.GUILIB.guiRegistry(:,1:idx-1) data.GUILIB.guiRegistry(:,idx+1:size(data.GUILIB.guiRegistry,2)) ];
    data.GUILIB.guiEntries = data.GUILIB.guiEntries - 1;
end

function dumpGuiRegistry( data )
    for i=1:size(data.GUILIB.guiRegistry,2);
        hObject = data.GUILIB.guiRegistry{1,i};
        if ishandle(hObject)
            type = get(hObject,'type');
            if strcmp(type, 'uicontrol')
                type = get(hObject,'style');
                type = [ 'uicontrol(' type ')']; %#ok<AGROW>
            end
            tag  = get(hObject,'tag');
            parent = get( hObject, 'parent');
            handleInfo = [ 'handle(' num2str(hObject) ') type: ' type ', tag: ' tag ', parent: ' num2str(parent) ];
        else
            handleInfo = [ 'illegal handle(' num2str(hObject) ')' ];
        end
        display([ num2str(i) ': ' handleInfo ]);
    end
    
    for i=1:size(data.GUILIB.layout,1)
        printLayoutTable( data.GUILIB.layout{i,1} );
    end
end

function data = storeUpdateEntry( data, hObject, updates )
% create a new entry in the guiRegistry
% this list holds all properties who needs to be updated
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if ~isfield(data.GUILIB,'guiRegistry')
        data.GUILIB.guiRegistry = cell(6,0);
        data.GUILIB.guiEntries = 0;
        data.GUILIB.visible = cell(2,0);
        data.GUILIB.closeCallbacks = cell(2,0);
    end
    
    if isobject( hObject ) % timer
    else
        if ~ishandle( hObject ); display(hObject); throwError('hObject is not a handle '); end;
    end
    idx = findUpdateIndexByHandle( data, hObject );
    if ~isempty( idx ); throwError([ 'handle ' num2str(hObject) ' allready in gui update list']); end;
    data.GUILIB.guiEntries = data.GUILIB.guiEntries + 1;
    data.GUILIB.guiRegistry{1,data.GUILIB.guiEntries} = hObject;
    
    c = size(updates,2);
    if mod(c,2) ~= 0; throwError('array definition must be { "property name" "property value"}'); end;
    
    type = get( hObject, 'type');
    if strcmpi( type, 'uicontrol' ); style = get( hObject, 'style'); end;
    
    i = 1;
    maintainc = 0;
    maintained = cell(2,maintainc);
    standardc = 0;
    standard = cell(2,standardc);
    specialc = 0;
    special = cell(2,specialc);
    while i<=c
        current_update = lower(updates{1,i});
        %check property:
        if ~ischar(updates{1,i}); throwError('property name must be of type char');end;
        try
            switch type % check if property exists
                case 'hgjavacomponent'  % has no real properties
                    switch current_update
                        case {'treedata' 'treedyndata' 'multipleselectionenabled' 'dragdropenabled' 'text'}
                        otherwise
                            v = get( hObject, current_update ); %#ok<NASGU>
                    end
                case 'figure'
                    switch current_update % dont't check pure java properties
                        case { 'focusgainedcallback' 'focuslostcallback' 'keypressedcallback' 'keyreleasedcallback' ...
                                'keytypedcallback' 'mouseclickedcallback' 'mousedraggedcallback' ...
                                'mouseenteredcallback' 'mouseexitedcallback' 'mousemovedcallback' ...
                                'mousepressedcallback' 'mousereleasedcallback' 'mousewheelmovedcallback' ...
                                'propertychangecallback' }
                        otherwise
                            v = get( hObject, current_update ); %#ok<NASGU>
                    end
                case 'uicontrol'
                    switch current_update
                        case 'continuouse' % used for slider
                        otherwise
                            v = get( hObject, current_update ); %#ok<NASGU>
                    end
                case 'timer'
                    switch current_update
                        case {'deref','startstop'} % this is not a property of timer
                        otherwise
                            v = get( hObject, current_update ); %#ok<NASGU>
                    end
                case 'uitable'
                    switch current_update
                        case 'viewline' % used for set line
                        otherwise
                            v = get( hObject, current_update ); %#ok<NASGU>
                    end
                case 'uitabgroup'
                    switch current_update
                        case 'selectedindex'
                        case 'backgroundcolor'
                            if data.GUILIB.matlabVersion < 2014.5
                                v = get( hObject, current_update ); %#ok<NASGU>
                            end
                        otherwise
                            v = get( hObject, current_update ); %#ok<NASGU>
                    end
                otherwise
                    v = get( hObject, current_update ); %#ok<NASGU>
            end
        catch %#ok<CTCH>
            throwError( ['Crash while getting property "' updates{1,i} '" from type "' type '"' ]);
        end
        value = updates{1,i+1};
        
        switch type
            case 'uicontrol'
                switch style
                    case 'edit'
                        if strcmpi( updates{1,i}, 'string' )
                            dataIndx = findEntryIndexByName( data, value );
                            specialc = specialc + 1;
                            special{1,specialc} = { @doUpdateEditTextObject hObject value dataIndx};
                            data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = get( hObject, 'BackgroundColor');
                            i = i + 2;
                            continue;
                        end
                        if strcmpi( updates{1,i}, 'buttondownfcn' )
                            specialc = specialc + 1;
                            special{1,specialc} = { @doUpdateAxesCallback hObject value 0 };
                            i = i + 2;
                            continue;
                        end
                    case {'listbox','popupmenu'}
                        if strcmpi( updates{1,i}, 'value' )
                            dataIndx = findEntryIndexByName( data, value );
                            specialc = specialc + 1;
                            special{1,specialc} = { @doUpdateListboxObject hObject value dataIndx};
                            data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = get( hObject, 'BackgroundColor');
                            i = i + 2;
                            continue;
                        end
                    case {'slider'}
                        switch updates{1,i}
                            case 'value'
                                dataIndx = findEntryIndexByName( data, value );
                                specialc = specialc + 1;
                                special{1,specialc} = { @doUpdateSliderObject hObject value dataIndx};
                                data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = get( hObject, 'BackgroundColor');
                                i = i + 2;
                                continue;
                            case 'Continuouse'
                                dataIndx = findEntryIndexByName( data, value );
                                specialc = specialc + 1;
                                special{1,specialc} = { @doUpdateSliderContinueouse hObject value dataIndx};
                                data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = get( hObject, 'BackgroundColor');
                                i = i + 2;
                                continue;
                        end                        
                    otherwise
                end
            case 'uipanel'
                if strcmpi( updates{1,i}, 'SelectedObject' ) % radiogroup
                    dataIndx = findEntryIndexByName( data, value );
                    specialc = specialc + 1;
                    special{1,specialc} = { @doUpdateSelectedObject hObject value dataIndx };
                    i = i + 2;
                    data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = get( hObject, 'BackgroundColor');
                    continue;
                end            
            case 'uitable'
                data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = get( hObject, 'BackgroundColor');
                switch updates{1,i}
                    case 'CellSelectionCallback'
                        if isa( value, 'function_handle')
                            standardc = standardc + 1;
                            standard{1,standardc} = updates{1,i}; % one time update
                            standard{2,standardc} = { @callRedirect, @handleTableSelected, value };
                            i = i + 2;
                            continue;
                        else
                            throwError('CellSelectionCallback need function handle');
                        end
                    case 'data'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateTableObject hObject value dataIndx };
                        i = i + 2;
                        continue;
                    case 'ViewLine'
                        specialc = specialc + 1;
                        dataIndx = findEntryIndexByName( data, value );
                        try
                            jscrollpane = findjobj(hObject);
                            set(jscrollpane, 'AdjustmentValueChangedCallback', {@TableAdjustmentValueChangedCallback hObject dataIndx} );
                        catch
                            jscrollpane = [];
                        end
                        special{1,specialc} = { @doUpdateTableViewLine hObject value jscrollpane };
                        i = i + 2;
                        continue;
                    otherwise
                end
            case 'uitabgroup'
                if data.GUILIB.matlabVersion < 2014.5
                    data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = get( hObject, 'BackgroundColor');
                else
                    % FIXME get background color
                    j = get(hObject, 'UserData');
                    if ~isempty(j) && length(j) >= 3
                        data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = getBackgroundColorFromHandle(j(3));
                    end
                end
                switch updates{1,i}
                    case 'SelectedIndex'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateTabObject hObject value dataIndx };
                        i = i + 2;
                        continue;
                    otherwise
                end
            case 'hgjavacomponent'
                switch updates{1,i}
                    case 'TreeData'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateTreeData hObject value dataIndx };
                        i = i + 2;
                        continue;
                    case 'TreeDynData'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateTreeDynRoot hObject value dataIndx };
                        i = i + 2;
                        continue;
                    case 'MultipleSelectionEnabled'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateTreeMultiSelect hObject value dataIndx };
                        i = i + 2;
                        continue;
                    case 'DragDropEnabled'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateDragDrop hObject value dataIndx };
                        i = i + 2;
                        continue;
                    case 'Text'
                        %dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateHComponent hObject value updates{1,i} };
                        i = i + 2;
                        continue;
                    otherwise
                end
            case 'axes'
                switch updates{1,i}
                    case 'tag'
                        maintainc = maintainc + 1; % the stupid plot comand removes the tag from axes!
                        maintained{1,maintainc} = updates{1,i}; % prepare this property for automatic update
                        maintained{2,maintainc} = [ '''' value '''' ];
                        i = i + 2;
                        continue
                    case 'buttondownfcn'
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateAxesCallback hObject value 0 };
                        i = i + 2;
                        continue;
                end
            case 'figure'
                switch updates{1,i}
                    case 'CloseRequestFcn'
                        closecl = false;
                        for cl = 1:size(data.GUILIB.closeCallbacks,2)
                            if (hObject == data.GUILIB.closeCallbacks{1,cl})
                                data.GUILIB.closeCallbacks{1,cl} = hObject;
                                data.GUILIB.closeCallbacks{2,cl} = value;
                                closecl = true;
                                break;
                            end
                        end
                        if closecl==false
                            cl = size(data.GUILIB.closeCallbacks,2)+1;
                            data.GUILIB.closeCallbacks{1,cl} = hObject;
                            data.GUILIB.closeCallbacks{2,cl} = value;
                        end
                        i = i + 2;
                        continue
                    case { 'WindowButtonUpFcn' 'WindowButtonMotionFcn' 'WindowScrollWheelFcn'}
                        if isa( value, 'function_handle')
                            standardc = standardc + 1;
                            standard{1,standardc} = updates{1,i}; % one time update
                            standard{2,standardc} = { @callRedirect, @handleWindowXXX, value };
                            i = i + 2;
                            continue;
                        else
                            throwError([updates{1,i} ' need function handle']);
                        end
                    case { 'FocusGainedCallback' 'FocusLostCallback' 'KeyPressedCallback' 'KeyReleasedCallback' ...
                                'KeyTypedCallback' 'MouseClickedCallback' 'MouseDraggedCallback' ...
                                'MouseEnteredCallback' 'MouseExitedCallback' 'MouseMovedCallback' ...
                                'MousePressedCallback' 'MouseReleasedCallback' 'MouseWheelMovedCallback' ...
                                'PropertyChangeCallback' }
                        if isa( value, 'function_handle')
                            if data.GUILIB.matlabVersion < 2014.5
                                setFigureJavaFrame(hObject, updates{1,i}, value)
                            else
                                setFigureJavaFrame2014b(hObject, updates{1,i}, value)
                            end
                            i = i + 2;
                            continue;
                        else
                            throwError([updates{1,i} ' need function handle']);
                        end
                    case 'position'
                        for fc=1:size(data.GUILIB.figures,2)
                            if data.GUILIB.figures(fc)==hObject
                                % store data index in figure position entry
                                % list, we need this in the resize cb
                                posIndx = findEntryIndexByName( data, value );
                                if ~isempty(posIndx)
                                    data.GUILIB.figPosEntry(fc) = posIndx;
                                else
                                    data.GUILIB.figPosEntry(fc) = 0;
                                end
                                break;
                            end
                        end
                end
            case 'timer'
                switch updates{1,i}
                    case 'deref'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateDeref hObject value dataIndx };
                        i = i + 2;
                        continue;
                    case 'startstop'
                        dataIndx = findEntryIndexByName( data, value );
                        specialc = specialc + 1;
                        special{1,specialc} = { @doUpdateTimer hObject value dataIndx };
                        i = i + 2;
                        continue;
                end
            otherwise
        end
        
        if strcmpi( updates{1,i}, 'backgroundcolor' )
            dataIndx = findEntryIndexByName( data, value );
            if strcmp('hgjavacomponent', type) 
                specialc = specialc + 1;
                if data.GUILIB.matlabVersion < 2014.5
                    fc = get(hObject,'FigureChild');
                    fc = get(fc,'FigureComponent');
                else
                    fc = get(hObject, 'JavaPeer');
                end
                special{1,specialc} = { @doUpdateBackgroundColorJava fc value dataIndx };
                i = i + 2;
                continue;
            end
            
            if ( data.GUILIB.matlabVersion >= 2014.5 && strcmp('uitabgroup', type) )
                fc = get(hObject, 'UserData');
                if ~isempty(fc)
                    specialc = specialc + 1;
                    special{1,specialc} = { @doUpdateBackgroundColorJava fc value dataIndx };
                end
                i = i + 2;
                continue;
            end
            if data.GUILIB.matlabVersion >= 2014.5 && strcmp('uipanel', type) && ischar(value)
                if strcmp(value,'none')
                    warning('since matlab version 2014b a panel with backgroundcolor ''none'' is not allowed');
                    i = i + 2;
                    continue
                end
                
            end
            
            if ~isempty(dataIndx) 
                specialc = specialc + 1;
                special{1,specialc} = { @doUpdateBackgroundColor hObject value dataIndx };
                i = i + 2;
                continue;
                %throwError('variable backgroundcolor not supported');
            end
            standardc = standardc + 1;
            if isempty(value)
                display('datagui:storeUpdateEntry try to set empty backgroundcolor')
            end
            data.GUILIB.guiRegistry{5,data.GUILIB.guiEntries} = value;
            standard{1,standardc} = updates{1,i}; % one time update
            standard{2,standardc} = value;
            i = i + 2;
            continue;
        end
        
        if strcmpi( updates{1,i}, 'visible' )
            dataIndx = findEntryIndexByName( data, value );
            if ~isempty(dataIndx); 
                vcounter = size(data.GUILIB.visible,2)+1;
                data.GUILIB.visible{1,vcounter} = hObject;
                data.GUILIB.visible{2,vcounter} = value;
            end
        end
        
        if ischar(value)
            dataIndx = findEntryIndexByName( data, value ); % is in maintain data list ?
            if ~isempty( dataIndx )
                maintainc = maintainc + 1;
                maintained{1,maintainc} = updates{1,i}; % prepare this property for automatic update
                maintained{2,maintainc} = value;
                i = i + 2;
                continue;
            else
                standardc = standardc + 1;
                standard{1,standardc} = updates{1,i}; % one time update
                standard{2,standardc} = value;
                i = i + 2;
                continue;
            end
        end
        if iscell(value) || isnumeric(value) || islogical(value)
            standardc = standardc + 1;
            standard{1,standardc} = updates{1,i}; % one time update
            standard{2,standardc} = value;
            i = i + 2;
            continue;
        end
        display(value);
        throwError('not supported type');
        i = i + 2;
    end
    
    if ~isempty(maintained)
        s = dataToStruct(data, maintained);
        set( hObject, s ); 
    end
    if ~isempty(standard)
        prop = standard(1,:);
        valu = standard(2,:);
        set( hObject, prop, valu );
        standard = [];
    end
    if ~isempty( special )
        for n=1:size(special,2)
            c = special{1,n};
            data.GUILIB.guiRegistry{6,data.GUILIB.guiEntries} = c{1}(data, c{2} , c{3}, c{4} );
        end
    end

    data.GUILIB.guiRegistry{2,data.GUILIB.guiEntries} = maintained;
    data.GUILIB.guiRegistry{3,data.GUILIB.guiEntries} = standard;
    data.GUILIB.guiRegistry{4,data.GUILIB.guiEntries} = special;
    data.GUILIB.guiRegistry{6,data.GUILIB.guiEntries} = [];
end

function indx = findUpdateIndexByHandle( data, hObject )
    indx = [];
    for i=1:data.GUILIB.guiEntries
        if data.GUILIB.guiRegistry{1,i} == hObject
            indx = i;
            return;
        end
    end
end

% public data api

function [data,error] = manualUpdateEntry( data, entryname, value, callcb, updategui )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if ~ischar(entryname); throwError('manualUpdateEntry expects a string as second parameter'); end;
    dataIndex = findEntryIndexByName( data, entryname );
    if isempty(dataIndex); throwError([ entryname ' not found']); end;
    [data,error] = updateEntry( data, dataIndex, value, true );
    if callcb == true
        display('disabled feature datagui3:manualUpdateEntry with call_callback==true, this should be fixed');
        %TODO FIXME
        %[data updategui] = doEntryCallbackRedraw( data, struct('hObject', jTree.UIContainer, 'jObject',jTree, 'dataIndex',dataIndex,'valueString',data.GUILIB.dataRegistry{1,dataIndex}) );    
    end
    if ~isempty(updategui) && islogical(updategui) && updategui == true
        data = doGuiUpdate( data );
        drawnow update;
        pause(0.1);
    end
end

function data = addData( data, input )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if ~iscell(input); throwError( 'input must be a cell array' ); end;
    for i=1:size(input,1)
        in = input{i,1};
        if ~iscell(in); throwError([ 'input row ' num2str(i) ' must be a cell array' ]); end;
        switch size(in,2)
            case 2
                data = insertEntry( data, in{1,1}, in{1,2} );
            case 3
                data = insertEntry( data, in{1,1}, in{1,2}, in{1,3} );
            case 4
                data = insertEntry( data, in{1,1}, in{1,2}, in{1,3}, in{1,4} );
            otherwise
                display(in);
                throwError('need 2-4 parameter');
        end
    end
end

function data = insertEntry( data, dataName, defaultValue, constrain, callback )
    if ~isstruct(data); throwError('data is not a struct'); end;
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if nargin<3 || nargin > 5; throwError( 'insertEntry need 3-5 parameter' ); end;
    if ~isfield(data,'GUILIB')
        data.GUILIB = struct();
    end
    if ~isfield(data.GUILIB,'dataRegistry')
        % format of dataRegistry:
        % 1: dataName
        % 2: default value
        % 3: cell array with index of linked gui elements (unused)
        % 4: type of entry (0:numeric, 1:string, 2:cell, 3:logical, 4:checked by constrain)
        % 5: constrain (NULL; [min max]; function call)
        % 6: callback
        data.GUILIB.dataRegistry = cell(6,0); 
        data.GUILIB.dataEntries = 0;
    end
    newIndx = findEntryIndexByName( data, dataName );
    if ~isempty( newIndx ); throwError( [dataName ' is allready maintained']); end;
    % create new entry:
    data.GUILIB.dataEntries = data.GUILIB.dataEntries + 1;
    data.GUILIB.dataRegistry{1,data.GUILIB.dataEntries} = dataName;
    % check input type of default parameter
    if isnumeric(defaultValue) 
        data.GUILIB.dataRegistry{4,data.GUILIB.dataEntries} = 0;
    else
        if ischar(defaultValue)
            data.GUILIB.dataRegistry{4,data.GUILIB.dataEntries} = 1;
        else
            if iscell(defaultValue)
                data.GUILIB.dataRegistry{4,data.GUILIB.dataEntries} = 2;
            else
                if islogical(defaultValue)
                    data.GUILIB.dataRegistry{4,data.GUILIB.dataEntries} = 3;
                else
                    throwError('unknown data type');
                end
            end
        end
    end
    
    if ~ischar(dataName); throwError('dataName must be a string'); end;
    if isempty(regexp(dataName,'^data.','ONCE')); throwError('dataName must start with "data."'); end;
    
    if nargin > 3
        if isnumeric(constrain) && ~isempty(constrain)
            if  data.GUILIB.dataRegistry{4,data.GUILIB.dataEntries} ~= 0
                throwError('min/max constrain without numeric default parameter');
            end
            if size(constrain,2) ~= 2
                throwError('min/max constrain need 2 parameters');
            end
        else
            if ~isa(constrain,'function_handle') && ~isempty(constrain)
                throwError('constrain must be NULL, [min max] or function handle');
            end
        end
    end
    
    if nargin>=4 && ~isempty(constrain) && isa(constrain,'function_handle')
        % replace default check if a function as a constrain is defined
        data.GUILIB.dataRegistry{4,data.GUILIB.dataEntries} = 4;
    end
    
    data.GUILIB.dataRegistry{2,data.GUILIB.dataEntries} = defaultValue;
    
    switch nargin
        case 3
        case 4
            data.GUILIB.dataRegistry{5,data.GUILIB.dataEntries} = constrain;
        case 5
            data.GUILIB.dataRegistry{5,data.GUILIB.dataEntries} = constrain;
            data.GUILIB.dataRegistry{6,data.GUILIB.dataEntries} = callback;
        otherwise
            throwError('insertEntry internal error');
    end
    eval([dataName '= defaultValue;']); % push default value into data structure
    
end

function constrain = getConstrain(data, dataIndex)
    constrain = data.GUILIB.dataRegistry{5,dataIndex};
end

function data = updateConstrain( data, entryname, constrain )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    dataIndex = findEntryIndexByName( data, entryname );
    if isempty(dataIndex); throwError([ entryname ' not found']); end;
    data.GUILIB.dataRegistry{5,dataIndex} = constrain;
end

function [data,error] = updateEntry( data, dataIndex, value, convert )
    [data,value,error] = checkEntry( data, dataIndex, value, convert ); %#ok<ASGLU>
    if isempty(error)
        %if convert % now we convert data inside checkEntry
        %    switch data.GUILIB.dataRegistry{4,dataIndex}
        %    case 0
        %        if ~isnumeric(value)
        %            value = str2double(value); %#ok<NASGU>
        %        end
        %    end
        %end
        eval([data.GUILIB.dataRegistry{1,dataIndex} '= value;']); % set new value
    end
end

function [data,value,error] = checkEntry( data, dataIndex, value, convert )
    error = [];
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    if dataIndex > data.GUILIB.dataEntries; throwError('dataIndex > dataEntries'); end;
    % check if value has the same type as the default value
    switch data.GUILIB.dataRegistry{4,dataIndex}
        case 0
            if ~isnumeric(value)
                error='need numeric as input'; 
                if convert == true
                    value = str2double(value);
                    if isnan( value ); return; end;
                    error = [];
                else
                    return; 
                end
            end
        case 1
            if ~ischar(value); error='need char as input'; return; end
        case 2
            if ~iscell(value); 
                error='need cell array as input'; 
                return; 
            end
        case 3
            if ~islogical(value); error='need logical as input'; return; end
        case 4
            % checked by contrain
        otherwise
            throwError('internal updateEntry error');
    end
    % check constrain if defined
    if ~isempty(data.GUILIB.dataRegistry{5,dataIndex})
        constrain = data.GUILIB.dataRegistry{5,dataIndex};
        if isnumeric(constrain) 
            if constrain(1) > value
                error = [ 'min is ' num2str(constrain(1)) ' but value is ' num2str(value) ];
                return;
            end
            if constrain(2) < value
                error = [ 'max is ' num2str(constrain(2)) ' but value is ' num2str(value) ];
                return;
            end
        else
            [data,ret_value,error] = constrain(data, value, error); % user callback
            if ~isempty(error); 
                return; 
            end;
            if convert==true
                value = ret_value;
            end
        end
    end
end

function data = setDefaultValue( data, variableName )
    if nargout ~= 1; throwError( 'do not revert return data structure' ); end;
    if nargin ~= 2; throwError( 'need 2 parameters: data structure, variableName' ); end;
    if ~ischar(variableName); throwError('variableName must be of type char'); end;
    indx = findEntryIndexByName( data, variableName );
    if isempty(indx) 
        throwError(['variable ' variableName ' not found' ]);
    end
    eval([ variableName '=data.GUILIB.dataRegistry{2,indx};']);
end

function data = saveData( data, filename, filter)
    if nargin<2 && nargin>3; throwError( 'need 2 or 3 parameters: data structure, filename, (filter)' ); end;
    if nargout ~= 1; throwError( 'do not revert return data structure' ); end;
    
    store = struct(); %#ok<NASGU>
    if nargin == 3    
        if ~iscell(filter); throwError('filter list must be a cell array of strings'); end
        for i=1:data.GUILIB.dataEntries
            for n=1:length(filter)
                if strcmp(char(data.GUILIB.dataRegistry{1,i}),filter{n})
                    %display([char(data.GUILIB.dataRegistry{1,i}) ' '])
                    eval([ 'store.s.' char(data.GUILIB.dataRegistry{1,i}) ...
                        '=' char(data.GUILIB.dataRegistry{1,i}) ';' ]);
                    break;
                end
            end
        end
    else
        for i=1:data.GUILIB.dataEntries
            %display([char(data.GUILIB.dataRegistry{1,i}) ''])
                    eval([ 'store.s.' char(data.GUILIB.dataRegistry{1,i}) ...
                        '=' char(data.GUILIB.dataRegistry{1,i}) ';' ]);
        end
    end
    eval([ 'save ' filename ' ''-struct'' store ; ' ]);
end

function data = loadData( data, filename )
    if nargin ~= 2; throwError( 'need 2 parameters: data structure, filename' ); end;
    if nargout ~= 1; throwError( 'do not revert return data structure' ); end;
    eval([ 'load ' filename  ]);
    names = fieldnames(s.data);
    for i=1:size(names,1)
        eval([ 'data.' char(names(i,1)) '= s.data.' char(names(i,1)) ';' ]);
    end
end

function data = registerDataErrorCallback(data, callbackHandler)
    if nargin ~= 2; throwError( 'need 2 parameters: data structure, callbackHandler' ); end;
    if nargout ~= 1; throwError( 'do not revert return data structure' ); end;
    data.GUILIB.errorCB = callbackHandler;
end

function [data,redraw] = doErrorCallback( data, event )
    cb = data.GUILIB.errorCB;
    if ~isempty(cb)
        [data,redraw]= cb( data, [], event ); % user callback
    else
        redraw = false;
    end
end

% private data functions
function [data,redraw] = doEntryCallbackRedraw( data, event )
% called from textbuttonpressed, uicontrolValueChanged etc. and call cb from data
% the callback function can decide, if the gui should be redrawn
% default: redraw
    if isnumeric(event.dataIndex)
        cb = data.GUILIB.dataRegistry{6,event.dataIndex};
    else
        cb = event.dataIndex;
        event.dataIndex = [];
    end
    redraw = [];
    if ~isempty( cb )
        try
            data = updateFigurePositionDataEntry(data);
            if iscell( cb ) 
                switch size( cb, 2)
                    case 1
                        [data,redraw]= cb{1}( data, [], event ); % user callback
                    case 2
                        [data,redraw] = cb{1}( data, [], event, cb{2} ); % user callback
                    case 3
                        [data,redraw] = cb{1}( data, [], event, cb{2}, cb{3} ); % user callback
                    case 4
                        [data,redraw] = cb{1}( data, [], event, cb{2}, cb{3}, cb{4} ); % user callback
                    case 5
                        [data,redraw] = cb{1}( data, [], event, cb{2}, cb{3}, cb{4}, cb{5} ); % user callback
                    otherwise
                        redraw = false;
                        throwError([ 'callback function with ' num2str(size( cb, 2)) ' parameter is not supported' ]);
                end
            else
                [data,redraw]= cb( data, [], event ); % user callback
            end
        catch e
            if ~isempty(e.stack) && strcmp(e.stack(1).name,'doEntryCallbackRedraw')
                if ~ishandle(event.hObject)
                    display(['hint: maybe function ''' char(cb) ''' not found. callback used in variable ''' data.GUILIB.dataRegistry{1,event.dataIndex} '''' ]);
                else
                    display(['hint: maybe function ''' char(cb) ''' not found. callback used in variable ''' data.GUILIB.dataRegistry{1,event.dataIndex} ''', gui tag: ' get(event.hObject,'tag')]);
                end
            end
            shutdownDataGUI(e, data.GUILIB.isDesktop);
        end 
    end
end

function variableName = findNameByIndex( data, indx )
    variableName = [];
    if indx > 0 && indx <= data.GUILIB.dataEntries
        variableName = data.GUILIB.dataRegistry{1,indx};
    end
end

function indx = findEntryIndexByName( data, name )
    indx = [];
    for i=1:data.GUILIB.dataEntries
        if strcmp( data.GUILIB.dataRegistry{1,i}, name ) 
            indx = i;
            return;
        end
    end
end

% other data_gui functions

function throwError( message )
% throw exception
    isdesktop = false;
    try
        cmdWinDoc = com.mathworks.mde.cmdwin.CmdWinDocument.getInstance;
        listeners = cmdWinDoc.getDocumentListeners;
        if length(listeners) > 1; isdesktop = true; end
    catch %#ok<CTCH>
        isdesktop = true;
    end
    ME = MException('data_gui:fatal', message );
    try
        throw(ME); % we use rethrow in shutdownDataGUI, so we have to throw an exception
    catch exp
        shutdownDataGUI(exp, isdesktop);
    end
end

function shutdownDataGUI(currentException, isDesktop)
    if ~isempty(get(0,'CurrentFigure')) % show dialog only if a figure is open
        t = timerfindall();
        if ~isempty(t); 
            stop(t)
            delete(t); 
        end
        try
            txt = cell(0,1);
            txt{size(txt,1)+1,1} = 'Unhandled error in matlab function detected:';
            txt{size(txt,1)+1,1} = '';
            txt{size(txt,1)+1,1} = currentException.message;
            txt{size(txt,1)+1,1} = '';
            txt{size(txt,1)+1,1} = 'Stack trace:';
            for n=1:length(currentException.stack)
                [~,name,ext] = fileparts(currentException.stack(n).file);
                txt{size(txt,1)+1,1} = [name ext '(' num2str(currentException.stack(n).line) '):' currentException.stack(n).name];
            end
            txt{size(txt,1)+1,1} = '';
            txt{size(txt,1)+1,1} = 'The matlab session will be terminated!';
            h = msgbox(txt ,'Exception detected','error','modal');
            set(h,'Color',[1 1 1]);
            c = get(h,'children');
            for i=1:length(c)           
                if strcmp(get(c(i),'Tag'), 'OKButton')
                    fp = get(h,'position');
                    p = get(c(i),'position');
                    p(1) = 1; p(3) = fp(3)-2;
                    set(c(i),'position', p, 'String', 'Terminate now!','BackgroundColor', [ 1 0 0]);
                    break; 
                end
            end
            uiwait(h);
        catch
        end
        close all;
    end 
    if isDesktop     
        rethrow(currentException); 
    else
        exit(2);
    end
end

% layout gedoens:

function data = addLayout( data, fig, layout_in )
    if nargout~=1; throwError( 'do not revert returned data structure' ); end;
    index = [];
    if ischar(fig)
        fig = findobj(data.GUILIB.figures,'tag',fig);
    end
    if ishandle(fig)
        for i=1:size(data.GUILIB.figures,2)
            if fig == data.GUILIB.figures(i)
                index = i;
                break;
            end
        end
    end
    if isempty(index); throwError(['figure ' num2str(fig) ' not found']); end;
    
    hiddenStatus = get(0,'ShowHiddenHandles');
     % somewhere in the createLayoutTable is a bug if hidden handles are
     % visible and tab pages are in the layout
    set(0,'ShowHiddenHandles','off')
    children = getAllChildren( fig );
    [ layout,evalResize ] = createLayoutTable( layout_in, fig, [], [], 0, '', children);
    set(0,'ShowHiddenHandles',hiddenStatus)
    
    evalResize = addVisibleToEvalResize( evalResize, layout, data.GUILIB.visible );
    data = registerDependendLayoutVariables(data, fig, layout_in);
    
    if ~isfield( data.GUILIB, 'layout' )
        data.GUILIB.layout = cell(1);
        data.GUILIB.evalResize = cell(1);
        data.GUILIB.celllayout = cell(1); % cell format, backup for layout editor
    end
    
    data.GUILIB.layout{index,1} = layout;
    data.GUILIB.celllayout{index,1} = layout_in;
    data.GUILIB.evalResize{index,1} = evalResize;
    set( fig, 'ResizeFcn', { @figureResize2 index }, 'Resize', 'on' );
    storeUserData(data);
    figureResize2(fig, [], index);
    % TODO, quick bugfix:
    g = get(fig,'children');
    if ~isempty(g)
        if strcmp(get(g(1),'type'), 'uicontrol') && ...
            strcmp(get(g(1),'style'), 'pushbutton') && ...
            length(g) > 1
            c = g;
            c(1) = g(2);
            c(2) = g(1);
            set(fig,'children',c);
        end
    end
end

function hasZero = layoutParameterCheck( space, inSize, inLayout )
    if ~isnumeric(space)
        throwError('expexts numeric as first parameter');
    end
    if ~iscell(inSize) || ~iscell(inLayout)
        throwError('expexts cell arrays as second and 3rd parameter');
    end
    if size(inSize,2) ~= size(inLayout,2)
        throwError('arrays of input size and layout destination must have the same dimension');
    end
    if nargout > 0
        hasZero = false;
        for i=1:size(inSize,2)
            if inSize{1,i} == 0
                hasZero = true;
                return;
            end
        end
    end
end

function out = layoutPanel( inHandles, inLayout )
    out = cell(1,2);
    if size(inHandles,2) ~= size(inLayout,2)
        throwError('arrays of input size and layout destination must have the same dimension');
    end
    for i=1:size(inHandles,2)
        if ~( ischar(inHandles{1,i}) || ishandle(inHandles{1,i}) )
            throwError('first cell array need handles or tag-strings');
        end
        if ~iscell( inLayout{1,i} )
            if ischar(inLayout{1,i})
                inLayout{1,i} = { { 0 }; inLayout(1,i) };
            else
                throwError('second array need cell array or tag-string');
            end
        end
    end
    out{1,1} = inHandles;
    out{1,2} = inLayout;    
end

function out = layoutTop( space, inSize, inLayout )
    layoutParameterCheck( space, inSize, inLayout );
    out = cell(2,1);
    if space <= 0
        if inSize{1,1} == 0 % do not add padding 0 is first entry size is 0
            out{1,1} = inSize;
            out{2,1} = inLayout;
        else
            out{1,1} = [0 inSize ];
            inL = [' ' inLayout];
            inL{1,1} = [];
            out{2,1} = inL;
        end
        return;
    end
    outL = layoutPad( inLayout );
    outS = layoutHelper( inSize, 0, space, space );
    out{1,1} = outS;
    out{2,1} = outL;
end

function out = layoutBottom( space, inSize, inLayout )
    layoutParameterCheck( space, inSize, inLayout );
    out = cell(2,1);
    if space <= 0
        out{1,1} = inSize ;
        out{2,1} = inLayout;
        return;
    end
    outL = layoutPad( inLayout );
    outS = layoutHelper( inSize, space, space, 0 );
    outL = outL(1,1:size(outL,2)-1); % remove last entry
    outS = outS(1,1:size(outS,2)-1); % remove last entry
    out{1,1} = outS;
    out{2,1} = outL;
end

function out = layoutVCenter( space, inSize, inLayout )
    hasZero = layoutParameterCheck( space, inSize, inLayout );
    out = cell(2,1);
    if space <= 0
        if hasZero
            out{1,1} = inSize;
            out{2,1} = inLayout;
            return
        end
        out{1,1} = [0 inSize 0];
        inL = [' ' inLayout ' '];
        inL{1,1} = [];
        inL{1,size(inL,2)} = [];
        out{2,1} = inL;
        return;
    end
    outL = layoutPad( inLayout );
    if hasZero 
        outS = layoutHelper( inSize, space, space, space );
    else
        outS = layoutHelper( inSize, 0, space, 0 );
    end
    out{1,1} = outS;
    out{2,1} = outL;
end

function out = layoutHCenter( space, inSize, inLayout )
    hasZero = layoutParameterCheck( space, inSize, inLayout );
    out = cell(1,2);
    if space <= 0
        out{1,1} = [0 inSize 0];
        inL = [' ' inLayout ' '];
        inL{1,1} = [];
        inL{1,size(inL,2)} = [];
        out{1,2} = inL;
        return;
    end
    outL = layoutPad( inLayout );
    if hasZero 
        outS = layoutHelper( inSize, space, space, space );
    else
        outS = layoutHelper( inSize, 0, space, 0 );
    end
    out{1,1} = outS;
    out{1,2} = outL;
end

function out = layoutLeft( space, inSize, inLayout )
    layoutParameterCheck( space, inSize, inLayout );
    out = cell(1,2);
    if space <= 0
        out{1,1} = inSize ;
        out{1,2} = inLayout;
        return;
    end
    outL = layoutPad( inLayout );
    outS = layoutHelper( inSize, space, space, 0 );
    outL = outL(1,1:size(outL,2)-1); % remove last entry
    outS = outS(1,1:size(outS,2)-1); % remove last entry
    out{1,1} = outS;
    out{1,2} = outL;
end

function out = layoutRight( space, inSize, inLayout )
    layoutParameterCheck( space, inSize, inLayout );
    out = cell(1,2);
    if space <= 0
        out{1,1} = [0 inSize ];
        inL = [' ' inLayout];
        inL{1,1} = [];
        out{1,2} = inL;
        return;
    end
    outL = layoutPad( inLayout );
    outS = layoutHelper( inSize, 0, space, space );
    out{1,1} = outS;
    out{1,2} = outL;
end

function layout = layoutSquare( dim, margin, boxes )
    if length(dim) > 1
        dim_x = dim(1);
        dim_y = dim(2);
    else
        dim_x = dim;
        dim_y = dim;
    end

    if margin > 0
        dim_y = dim_y * 2 + 1;
        dim_x = dim_x * 2 + 1;
    end
    col_size = cell(1,dim_x);
    col_boxx = cell(1,dim_x);
    n = 1;
    for c=1:dim_x
        if mod(c,2) && margin > 0 % this is a margin entry
            col_size{1,c} = margin;
        else
            col_size{1,c} = 0;
            row_size = cell(1,dim_y);
            row_boxx = cell(1,dim_y);
            for r=1:dim_y
                if mod(r,2) && margin > 0 % this is a margin entry
                    row_size{1,r} = margin;
                else
                    row_size{1,r} = 0;
                    if n <= size(boxes,2)
                        row_boxx(1,r) = boxes(1,n);
                    end
                    n = n + 1;
                end
            end
            col_boxx{1,c} = { row_size row_boxx };
        end
    end
    layout = { col_size ; col_boxx };    
end

function out = layoutPad( inLayout )
    out=cell(1,size(inLayout,2)*2+1);
    n = 0;    
    for i=1:size(inLayout,2)
        n = n + 2;
        out(1,n) = inLayout(1,i) ;
    end
end

function result = layoutShrink( margin, in )
    result   = { { margin 0 margin } { [] { { margin 0 margin };{[] in [] } } [] } };
end

function out = layoutHelper( inSize, first, middle, last )
    s = size(inSize,2)*2+1;
    out=cell(1,s);
    n = 1;
    out{1,n} = first;
    
    for i=1:size(inSize,2)
        n = n + 1;
        out(1,n) = inSize(1,i) ;
        n = n + 1;
        out{1,n} = middle;
    end
    out{1,s} = last;
end

function s = a2str( in )
    s = ''; sep = '';
    if iscell(in)
        s = '{...}';
        return
    end
    if ~isempty(in)
        for i=1:length(in)
            s = [ s sep num2str(in(i)) ]; %#ok<AGROW>
            sep = ' ';
        end
    end
    s = [ '[' s ']' ];
end

function printLayoutTable( out )
    for i=1:size(out,1)
        switch out{i,1}
            case 0
                s = 'set ';
            case 1
                s = 'hor ';
            case 2
                s = 'ver ';
            case 3
                s = 'pan ';
            case 5
                s = 'nop ';
            otherwise
                s = [ 'unknown:' num2str(out{i,1}) ' '];
        end
        if isempty(out{i,2})
            s2 = '[]';
        else
            if isnumeric(out{i,2})
                s2 = num2str(out{i,2});
                if ishandle(out{i,2})
                    tag = get(out{i,2},'tag');
                    if ~isempty(tag)
                        s2 = tag;
                    end
                end
            else
                if iscell(out{i,2})
                    s2 = ''; sep = '';
                    for n=1:size(out{i,2},2)
                        if ischar(out{i,2}{1,n})
                            s2 = [ s2 sep out{i,2}{1,n} ]; %#ok<AGROW>
                        else
                            s2a = num2str(out{i,2}{1,n});
                            if ishandle(out{i,2}{1,n})
                                tag = get(out{i,2}{1,n},'tag');
                                if ~isempty(tag)
                                    s2a = tag;
                                end
                            end
                            s2 = [ s2 sep s2a]; %#ok<AGROW>
                        end
                        sep = ';';
                    end
                    s2 = [ '{' s2 '}']; %#ok<AGROW>
                else
                    if ischar(out{i,2})
                        s2 = out{i,2};
                    end
                end
            end
        end
        if isempty(out{i,5})
            s5 = '[]';
        else
            s5 = num2str(out{i,5});
        end
        if isempty(out{i,6})
            s6 = '[]';
        else
            s6 = ''; sep = '';
            for n=1:size(out{i,6},1)
                a = out{i,6};
                s6 = [ s6 sep num2str(a(n,1)) ' ' num2str(a(n,2)) ' ' num2str(a(n,3)) ' ' num2str(a(n,4))]; %#ok<AGROW>
                sep = ';';
            end
            s6 = [ '[' s6 ']' ]; %#ok<AGROW>
        end
        s8 = num2str(out{i,8});
        if ishandle(out{i,8})
            tag = get(out{i,8},'tag');
            if ~isempty(tag)
                s8 = tag;
            end
        end

        display([ num2str(i) ': ' s ' ' s2 ' ' a2str(out{i,3}) ' ' a2str(out{i,4}) ...
            ' ' num2str(out{i,7}) ' ' s5 ' ' s8 ' ' s6]);
    end
end


function [out,evalResize] = createLayoutTable( o, parent, visibleList, out, calling, evalResize, currentGuiHandles )
% called after figure resize, uses data.layout to modify gui elements
    cl_debug = false;
    if cl_debug; display([ 'createLayoutTable(' num2str(size(out,1)) ')  parent: ' num2str(parent) ' calling: ' num2str(calling) ]); end %#ok<UNRCH>

    if isempty( out ) 
        out = cell(0,9);
    end
    current_out = size(out,1);
    if isempty(o) % special case: none element in tabgroup
        current_out = current_out + 1;
        out{current_out,1} = 5; % mode: set nop
        out{current_out,2} = [];
        out{current_out,5} = [];
        out{current_out,6} = zeros(1,4);
        out{current_out,7} = calling;
        out{current_out,8} = parent;
        out{current_out,9} = [];
        if calling ~= 0
            out{calling,4}(out{calling,5}) = current_out;
            out{calling,5} = out{calling,5} + 1;
        end
        return
    end
    if ischar(o) % spezial case: one ui element in (sub) panel
        handles = unique(findobj(currentGuiHandles,'tag',o));
        if isempty(handles)
            throwError(['tag ' o ' not found']);
        end
        current_out = current_out + 1;
        out{current_out,1} = 0; % mode: set handle
        out{current_out,2} = handles;
        out{current_out,5} = [];
        out{current_out,6} = zeros(1,4);
        out{current_out,7} = calling;
        out{current_out,8} = parent;
        out{current_out,9} = 'on';
        if calling ~= 0
            out{calling,4}(out{calling,5}) = current_out;
            out{calling,5} = out{calling,5} + 1;
        end
        return
    end
    if ~isempty(o{1,1}) && ~iscell(o{1,1})
        if cl_debug; display([ 'createLayoutTable create nop (single spacer in uitab/panelmember' '']); end %#ok<UNRCH>
        current_out = current_out + 1;
        out{current_out,1} = 5; % mode: set nop
        out{current_out,2} = [];
        out{current_out,5} = [];
        out{current_out,6} = zeros(1,4);
        out{current_out,7} = calling;
        out{current_out,8} = parent;
        out{current_out,9} = [];
        if calling ~= 0
            out{calling,4}(out{calling,5}) = current_out;
            out{calling,5} = out{calling,5} + 1;
        end
        return
    end
    if ~isempty(o{1,1}) && ischar(o{1,1}{1,1})
            if cl_debug; display([ 'createLayoutTable uiplanel: '  o{1,1}{1,1} ]); end %#ok<UNRCH>
            current_out = current_out + 1;
            sizes = size(o{1,1},2);
            out{current_out,1} = 3; % mode: panel
            for l=1:length(o{1,1})
                if ischar(o{1,1}{l})
                    handle = unique(findobj(currentGuiHandles,'tag',o{1,1}{l} ));
                    if isempty( handle )
                        throwError([ 'tag ' char(o{1,1}{l}) ' not found']);
                    end
                    if l==1 && strcmp(get(handle,'type'),'uitabgroup')
                        tabgroup = handle;
                        tabs = get(tabgroup,'children');
                        sizes = length(tabs);
                        out{current_out,1} = 4; % mode: tab
                        out{current_out,2} = tabgroup; % tabgroup handle
                        out{current_out,4} = zeros(sizes,1); % content of tabs
                        out{current_out,5} = 1; % local pc
                        out{current_out,6} = zeros(sizes,4); % adresses of sub tabs
                        out{current_out,7} = calling; % pc of parent
                        out{current_out,8} = parent; % parent handle
                        out{current_out,9} = 'on'; % visible status of tab group
                        parent_out = current_out;
                        if calling ~= 0 % setup parent jump table entry
                            out{calling,4}(out{calling,5}) = current_out;
                            out{calling,5} = out{calling,5} + 1;
                        end
                        for m=1:sizes
                            parent2 = tabs(m);
                            p = o{1,2}{1,m};
                            [out,evalResize] = createLayoutTable( p, parent2, visibleList, out, parent_out, evalResize,currentGuiHandles );
                        end

                        return;
                    end
                    o{1,1}{l} = handle;
                end
            end
            out{current_out,2} = o{1,1};
            out{current_out,4} = zeros(sizes,1);
            out{current_out,5} = 1;
            out{current_out,6} = zeros(sizes,4);
            out{current_out,7} = calling;
            out{current_out,8} = parent;
            out{current_out,9} = cell(1,length(o{1,1}));
            for l=1:length(o{1,1})
                out{current_out,9}{1,l} = 'on';
            end
            parent_out = current_out;
            if calling ~= 0
                out{calling,4}(out{calling,5}) = current_out;
                out{calling,5} = out{calling,5} + 1;
            end
            
            for l=1:size(o{1,1},2)
                parent2 = o{1,1}{1,l};
                p = o{1,2}{1,l};
                [out,evalResize] = createLayoutTable( p, parent2, visibleList, out, parent_out, evalResize,currentGuiHandles );
            end
            return;
    end

    if size(o,1) < size(o,2) % horizontal or vertical
        if cl_debug; dbg_str = 'horizontal'; end %#ok<UNRCH>
        pointers = o{1, 2};
        set_mode = 1;
    else
        if cl_debug; dbg_str = 'vertical'; end %#ok<UNRCH>
        pointers = o{2, 1};
        set_mode = 2;
    end            
            
    if cl_debug; display(['createLayoutTable ' dbg_str ', calling: ' num2str(calling) ]); end %#ok<UNRCH>
    sizes = o{1, 1};
    [sizes,evalResize ] = CreateEvalStringForResize(sizes,current_out,evalResize);

    current_out = current_out + 1;
    out{current_out,1} = set_mode;
    out{current_out,3} = cell2mat(sizes)';
    out{current_out,4} = zeros(length(sizes),1);
    out{current_out,5} = 1;
    out{current_out,6} = zeros(length(sizes),4);
    out{current_out,7} = calling;
    out{current_out,8} = parent;
    out{current_out,9} = 'on';
    parent_out = current_out;
    if calling ~= 0
        out{calling,4}(out{calling,5}) = current_out;
        out{calling,5} = out{calling,5} + 1;
    end

    for l=1:size(pointers,2)
        p = pointers{1,l};
        if iscell(p)
            out{parent_out,5} = l;
            [out,evalResize] = createLayoutTable( p, parent, visibleList, out, parent_out, evalResize,currentGuiHandles );
            current_out = size(out,1);
        else
            if ischar(p)
                if cl_debug; display(['createLayoutTable element ' dbg_str ', ' p ', calling: ' num2str(calling) ', parent_out: ' num2str(parent_out) ' l:' num2str(l) ]); end %#ok<UNRCH>
                handles = unique(findobj(currentGuiHandles,'tag',p));
                if isempty(handles)
                    throwError(['tag ' p ' not found']);
                end
                current_out = current_out + 1;
                out{current_out,1} = 0; % mode: set handle
                out{current_out,2} = handles;
                out{current_out,5} = [];
                out{current_out,6} = zeros(1,4);
                out{current_out,7} = parent_out;
                out{parent_out,4}(l) = current_out;
               if ~strcmp(get(parent,'type'),'axes')
                    out{current_out,8} = parent;
                else
                    out{current_out,8} = findValidParentInOutList(out, current_out);
                end
                out{current_out,9} = 'on';
            end                
        end
    end
end

function parent = findValidParentInOutList(out, currentIndx)
    parent = [];
    while currentIndx ~= 0
        type = get(out{currentIndx,8},'type');
        if ~isempty(type)
            switch type
                case { 'figure' 'uipanel' 'uitabgroup' }
                    parent = out{currentIndx,8};
                    return;
                otherwise
            end
        end
        currentIndx = out{currentIndx,7};
    end
end

function [sizes,evalResize ] = CreateEvalStringForResize(sizes,current_out, evalResize)
    [x,y] = size(sizes);
    if x==1
        for i=1:y
            if ischar(sizes{1,i})
                evalResize = [ evalResize 'layout{' num2str(current_out+1) ',3}(' num2str(i) ') = ' sizes{1,i} ';']; %#ok<AGROW>
                sizes{1,i} = 0;
            end
        end
    end
    if y==1
        for i=1:x
            if ischar(sizes{i,x})
                evalResize = [ evalResize 'layout{' num2str(current_out+1) ',3}(' num2str(i) ') = ' sizes{i,1} ';']; %#ok<AGROW>
                sizes{i,1} = 0;
            end
        end
    end
end

function layout = sampleResize( layout, pos )
    % mode, handles, sizes, jump_list, local_pc, precalc_sizes, return_jump parent_handle visible
    % 1     2        3      4          5         6              7           8             9

    pc = 1; % program counter
    npc = pc;
    max_pc = size(layout,1);
    for r=1:max_pc; layout{r,5} = 1; end % reset all local program counter
    
    while ( pc > 0 && pc <= max_pc )
        switch layout{pc,1}
            case 0
                layout{pc,6} = pos;
                npc = layout{pc,7};
            case 1
                l = length(layout{pc,4});
                if layout{pc,5} > l
                    npc = layout{pc,7}; % return
                else
                    if layout{pc,5} == 1  % first enter
                        layout{pc,6} = resizeCalcH(layout{pc,3}, pos );
                    end
                    pos = layout{pc,6}(layout{pc,5},:);
                    if layout{pc,4}(layout{pc,5}) ~= 0 % skip empty entries
                        npc = layout{pc,4}(layout{pc,5});
                    end
                    layout{pc,5} = layout{pc,5} + 1;
                end
            case 2
                l = length(layout{pc,4});
                if layout{pc,5} > l
                    npc = layout{pc,7}; % return
                else
                    if layout{pc,5} == 1 % first enter
                        layout{pc,6} = resizeCalcV(layout{pc,3}, pos );
                    end
                    pos = layout{pc,6}(layout{pc,5},:);
                    if layout{pc,4}(layout{pc,5}) ~= 0 % skip empty entires
                        npc = layout{pc,4}(layout{pc,5});
                    end
                    layout{pc,5} = layout{pc,5} + 1;
                end
            case 3 % panel
                l = length(layout{pc,4});
                if layout{pc,5} > l
                    npc = layout{pc,7}; % return
                else
                    if layout{pc,5} == 1 % shrink only if on first enter
                        layout{pc,6} = pos;
                        %pos = [ pos(1) pos(2) pos(3)-4 pos(4)-17 ];
                        pos = [ 1 1 pos(3)-4 pos(4)-17 ];
                    else
                        pos = layout{pc,6};
                        pos = [ 1 1 pos(3)-4 pos(4)-17 ];
                    end
                    if layout{pc,4}(layout{pc,5}) ~= 0
                        npc = layout{pc,4}(layout{pc,5});
                    end
                    layout{pc,5} = layout{pc,5} + 1;
                end
            case 4 % tabgroup
                l = length(layout{pc,4});
                if layout{pc,5} > l
                    npc = layout{pc,7}; % return
                else
                    cut = 25;
                    if ismac(); cut = 34; end
                    if layout{pc,5} == 1 % shrink only if on first enter
                        layout{pc,6} = pos;
                        pos = [ 1 1 pos(3)-4 pos(4)-cut ];
                    else
                        pos = layout{pc,6};
                        pos = [ 1 1 pos(3)-4 pos(4)-cut ];
                    end
                    if layout{pc,4}(layout{pc,5}) ~= 0
                        npc = layout{pc,4}(layout{pc,5});
                    end
                    layout{pc,5} = layout{pc,5} + 1;
                end
            case 5 % nop
                npc = layout{pc,7}; % return
        end % switch
        pc = npc;
    end
end

function pos = resizeCalcV( sizes, rect )
    l = length(sizes);
    z = sum(sizes(:)==0);
    pos = zeros( l, 4);
    div = rect(4)/100;
    if ( z > 0 )
        n = sizes(:)>1;
        p = sizes(:)>0 & sizes(:)<1;
        all = sum(sizes(n)) +sum(sizes(p)*div*100);
        zero_part = fix( (rect(4)-all) / z);
    end
    
    for i=1:l
        if sizes(i) > 1
            s = fix(sizes(i));
            pos(i,:) = [ rect(1) rect(2) rect(3) s ];
            rect(:)  = [ rect(1) rect(2)+s rect(3) rect(4)+s ];
        elseif sizes(i) > 0
            s = fix(sizes(i)*div*100);
            pos(i,:) = [ rect(1) rect(2) rect(3) s ];
            rect(:)  = [ rect(1) rect(2)+s rect(3) rect(4)+s ];
        elseif sizes(i) == 0
            pos(i,:) = [ rect(1) rect(2) rect(3) zero_part ];
            rect(:)  = [ rect(1) rect(2)+zero_part rect(3) rect(4)+zero_part ];            
        end
    end
end

function pos = resizeCalcH( sizes, rect )
    l = length(sizes);
    z = sum(sizes(:)==0);
    pos = zeros( l, 4);
    div = rect(3)/100;
    if ( z > 0 )
        n = sizes(:)>1;
        p = sizes(:)>0 & sizes(:)<1;
        all = sum(sizes(n)) +sum(sizes(p)*div*100);
        zero_part = fix( (rect(3)-all) / z);
    end
    
    for i=1:l
        if sizes(i) > 1
            s = fix(sizes(i));
            pos(i,:) = [ rect(1) rect(2) s rect(4) ];
            rect(:)  = [ rect(1)+s rect(2) rect(3)+s rect(4) ];
        elseif sizes(i) > 0
            s = fix(sizes(i)*div*100);
            pos(i,:) = [ rect(1) rect(2) s rect(4) ];
            rect(:)  = [ rect(1)+s rect(2) rect(3)+s rect(4) ];
        elseif sizes(i) == 0
            pos(i,:) = [ rect(1) rect(2) zero_part rect(4)  ];
            rect(:)  = [ rect(1)+zero_part rect(2) rect(3)+zero_part rect(4) ];            
        end
    end
end

function sampleMove( layout )
    max_pc = size(layout,1);
    for r=1:max_pc;
        switch layout{r,1}
            case {0,4}
                h = layout{r,2};
                parent = layout{r,8};
                pos = layout{r,6};
                type = get(h,'type');
                if strcmp( type, 'axes' )
                    set(h,'units', 'pixels');
                    inset = get( h, 'TightInset' );
                    pos = layout{r,6};
                    pos(1) = pos(1) + inset(1);
                    pos(2) = pos(2) + inset(2);
                    pos(3) = pos(3) - ( inset(3) + inset(1) );
                    pos(4) = pos(4) - ( inset(4) + inset(2) );
                end
                if pos(3) < 5 || pos(4) < 5
                    set(h,'visible','off');
                else
                    set(h, 'position', pos, 'visible', layout{r,9}, 'parent', parent, 'units', 'pixels');
                end
            case 3
                h = layout{r,2};
                if iscell(h)
                    h = [h{:}];
                end
                parent = layout{r,8};
                pos = layout{r,6};
                if pos(3) < 5 || pos(4) < 5
                    set(h,'visible','off');
                else
                    for i=1:length(h)
                        posprop = 'position';
                        if strcmp(get(h(i),'type'),'axes')
                            posprop = 'outerposition';
                        end
                        set(h(i), 'units', 'pixels', 'parent', parent, posprop, pos, 'visible', layout{r,9}{i});
                    end
                end
        end
    end
end

function figureResize2(hObject, eventdata, layoutIndex) %#ok<INUSL>
% this is registered inside the ResizeFcn property from a figure
    try
        data = getRoot( hObject );
        if ( layoutIndex <= size(data.GUILIB.layout, 1) )
            layout = data.GUILIB.layout{layoutIndex,1};
            evalResize = data.GUILIB.evalResize{layoutIndex,1};
            pos = get( hObject, 'Position' );
            if ~isempty(evalResize); eval(evalResize); end
            layout = sampleResize( layout, [0 0 pos(3) pos(4) ] );
            %printLayoutTable( layout )
            sampleMove( layout);
            data = updateAxesInset(data);
            storeUserData( data );
        else
            throwError([ 'figure ' num2str(hObject) ' has no layout def' ]);
        end
    catch %#ok<CTCH>
        exc = lasterror; %#ok<LERR>
        error = exc.message;
        display([ 'exception in figureResize2: ' error ]);
    end
end

function data = manualResize( data, figure )
    if nargout==0; throwError( 'do not revert return data structure' ); end;
    figure = resolveHandle( data, figure );
    resize_cb = get(figure, 'ResizeFcn');
    if iscell(resize_cb)
        if size(resize_cb,1) == 2
            layoutIndex = resize_cb{2,1};
            layout = data.GUILIB.layout{layoutIndex,1};
            evalResize = data.GUILIB.evalResize{layoutIndex,1};
            pos = get( figure, 'Position' );
            if ~isempty(evalResize); eval(evalResize); end
            layout = sampleResize( layout, [ 0 0 pos(3) pos(4) ] );
            sampleMove( layout );
            data = updateAxesInset(data);            
        end
    end
end
    
function evalResize = addVisibleToEvalResize( evalResize, layout, visible_list )
    for i=1:size(visible_list,2)
        search = visible_list{1,i};
        for r=1:size(layout,1)
            if isempty(search)
                break
            end
            handles = layout{r,2};
            if isempty( handles )
                continue;
            end
            if iscell( handles )
                for h=1:length(handles)
                    if handles{h}~=search
                        continue;
                    end
                    evalResize=[ evalResize 'layout{' num2str(r) ',9}{' num2str(h) '}=' visible_list{2,i} ';']; %#ok<AGROW>
                    search= [];
                    break
                end
                continue
            end
            if ishandle( handles )
                if handles~=search
                    continue;
                end
                evalResize=[ evalResize  'layout{' num2str(r) ',9}=' visible_list{2,i} ';']; %#ok<AGROW>
                search = []; %#ok<NASGU>
                break
            end
        end
    end
end
